@@ -1,15 +1,19 @@
 #ifndef NEWSBOAT_CACHE_H_
 #define NEWSBOAT_CACHE_H_
 
+#include <memory>
 #include <mutex>
 #include <sqlite3.h>
 #include <unordered_set>
 
 #include "configcontainer.h"
-#include "rss.h"
 
 namespace newsboat {
 
+class RssFeed;
+class RssIgnores;
+class RssItem;
+
 struct SchemaVersion {
 	unsigned int major, minor;
 
@@ -50,8 +54,8 @@ public:
 	void cleanup_cache(std::vector<std::shared_ptr<RssFeed>>& feeds);
 	void do_vacuum();
 	std::vector<std::shared_ptr<RssItem>> search_for_items(
-		const std::string& querystr,
-		const std::string& feedurl);
+			const std::string& querystr,
+			const std::string& feedurl);
 	std::unordered_set<std::string> search_in_items(
 		const std::string& querystr,
 		const std::unordered_set<std::string>& guids);
@@ -67,8 +71,7 @@ public:
 	unsigned int get_unread_count();
 	void mark_item_deleted(const std::string& guid, bool b);
 	void mark_feed_items_deleted(const std::string& feedurl);
-	void remove_old_deleted_items(const std::string& rssurl,
-		const std::vector<std::string>& guids);
+	void remove_old_deleted_items(RssFeed* feed);
 	void mark_items_read_by_guid(const std::vector<std::string>& guids);
 	std::vector<std::string> get_read_item_guids();
 	void fetch_descriptions(RssFeed* feed);
@@ -89,8 +92,8 @@ private:
 		const std::string& arg,
 		Args... args);
 	template<typename T, typename... Args>
-	std::string
-	prepare_query(const std::string& format, const T& arg, Args... args);
+	std::string prepare_query(const std::string& format, const T& arg,
+		Args... args);
 
 	void run_sql(const std::string& query,
 		int (*callback)(void*, int, char**, char**) = nullptr,
