@@ -2,6 +2,7 @@
 #define NEWSBOAT_CONFIGCONTAINER_H_
 
 #include <unordered_set>
+#include <mutex>
 
 #include "configparser.h"
 
@@ -18,7 +19,7 @@ enum class FeedSortMethod {
 	LAST_UPDATED
 };
 
-enum class ArtSortMethod { TITLE, FLAGS, AUTHOR, LINK, GUID, DATE };
+enum class ArtSortMethod { TITLE, FLAGS, AUTHOR, LINK, GUID, DATE, RANDOM };
 
 enum class SortDirection { ASC, DESC };
 
@@ -94,6 +95,7 @@ public:
 
 private:
 	std::map<std::string, ConfigData> config_data;
+	std::recursive_mutex config_data_mtx;
 
 	bool is_bool(const std::string& s);
 	bool is_int(const std::string& s);
