@@ -2,9 +2,8 @@
 #define NEWSBOAT_CONFIGPARSER_H_
 
 #include <map>
-#include <set>
-#include <string>
-#include <vector>
+
+#include "configactionhandler.h"
 
 namespace newsboat {
 
@@ -16,14 +15,6 @@ enum class ActionHandlerStatus {
 	FILENOTFOUND
 };
 
-struct ConfigActionHandler {
-	virtual void handle_action(const std::string& action,
-		const std::vector<std::string>& params) = 0;
-	virtual void dump_config(std::vector<std::string>& config_output) = 0;
-	ConfigActionHandler() {}
-	virtual ~ConfigActionHandler() {}
-};
-
 class ConfigParser : public ConfigActionHandler {
 public:
 	ConfigParser();
@@ -37,7 +28,7 @@ public:
 	{
 		/* nothing because ConfigParser itself only handles include */
 	}
-	bool parse(const std::string& filename, bool double_include = true);
+	bool parse(const std::string& filename);
 	static std::string evaluate_backticks(std::string token);
 
 private:
@@ -45,18 +36,7 @@ private:
 	static std::string evaluate_cmd(const std::string& cmd);
 	std::vector<std::vector<std::string>> parsed_content;
 	std::map<std::string, ConfigActionHandler*> action_handlers;
-	std::set<std::string> included_files;
-};
-
-class NullConfigActionHandler : public ConfigActionHandler {
-public:
-	NullConfigActionHandler() {}
-	~NullConfigActionHandler() override {}
-	void handle_action(const std::string&,
-		const std::vector<std::string>&) override
-	{
-	}
-	void dump_config(std::vector<std::string>&) override {}
+	std::vector<std::string> included_files;
 };
 
 } // namespace newsboat
