@@ -5,7 +5,7 @@
 
 namespace podboat {
 
-enum class dlstatus {
+enum class DlStatus {
 	QUEUED = 0,
 	DOWNLOADING,
 	CANCELLED,
@@ -17,24 +17,25 @@ enum class dlstatus {
 	PLAYED
 };
 
-class pb_controller;
+class PbController;
 
-class download {
+class Download {
 public:
-	explicit download(pb_controller* c = 0);
-	~download();
+	explicit Download(PbController* c = 0);
+	~Download();
 	double percents_finished() const;
 	const std::string status_text() const;
-	dlstatus status() const
+	DlStatus status() const
 	{
 		return download_status;
 	}
 	const std::string filename() const;
+	const std::string basename() const;
 	const std::string url() const;
 	void set_filename(const std::string& str);
 	void set_url(const std::string& url);
 	void set_progress(double downloaded, double total);
-	void set_status(dlstatus dls);
+	void set_status(DlStatus dls);
 	void set_kbps(double kbps);
 	double kbps() const;
 	void set_offset(unsigned long offset);
@@ -51,12 +52,12 @@ public:
 private:
 	std::string fn;
 	std::string url_;
-	dlstatus download_status;
+	DlStatus download_status;
 	float cursize;
 	float totalsize;
 	double curkbps;
 	unsigned long offs;
-	pb_controller* ctrl;
+	PbController* ctrl;
 };
 
 } // namespace podboat
