@@ -5,16 +5,16 @@
 
 namespace newsboat {
 
-typedef std::pair<std::string, std::string> filter_name_expr_pair;
+typedef std::pair<std::string, std::string> FilterNameExprPair;
 
-class filtercontainer : public config_action_handler {
+class FilterContainer : public ConfigActionHandler {
 public:
-	filtercontainer() {}
-	~filtercontainer() override;
+	FilterContainer() {}
+	~FilterContainer() override;
 	void handle_action(const std::string& action,
 		const std::vector<std::string>& params) override;
 	void dump_config(std::vector<std::string>& config_output) override;
-	std::vector<filter_name_expr_pair>& get_filters()
+	std::vector<FilterNameExprPair>& get_filters()
 	{
 		return filters;
 	}
@@ -24,7 +24,7 @@ public:
 	}
 
 private:
-	std::vector<filter_name_expr_pair> filters;
+	std::vector<FilterNameExprPair> filters;
 };
 
 } // namespace newsboat
