@@ -1,16 +1,18 @@
 #ifndef NEWSBOAT_FORMACTION_H_
 #define NEWSBOAT_FORMACTION_H_
 
+#include <memory>
 #include <string>
 #include <vector>
 
 #include "history.h"
 #include "keymap.h"
-#include "rss.h"
 #include "stflpp.h"
 
 namespace newsboat {
 
+class ConfigContainer;
+class RssFeed;
 class View;
 
 struct KeyMapHintEntry {
@@ -53,7 +55,7 @@ public:
 	std::string get_qna_response(unsigned int i)
 	{
 		return (qna_responses.size() >= (i + 1)) ? qna_responses[i]
-							 : "";
+			: "";
 	}
 	void start_qna(const std::vector<QnaPair>& prompts,
 		Operation finish_op,
