@@ -2,7 +2,6 @@
 #define NEWSBOAT_HISTORY_H_
 
 #include <string>
-#include <vector>
 
 namespace newsboat {
 
@@ -17,8 +16,7 @@ public:
 	void save_to_file(const std::string& file, unsigned int limit);
 
 private:
-	std::vector<std::string> lines;
-	unsigned int idx;
+	void* rs_hst = nullptr;
 };
 
 } // namespace newsboat
