@@ -29,9 +29,7 @@ public:
 	void set_redraw(bool b) override
 	{
 		FormAction::set_redraw(b);
-		apply_filter = !(v->get_cfg()->get_configvalue_as_bool(
-			"show-read-articles"));
-		invalidate(InvalidationMode::COMPLETE);
+		invalidate_everything();
 	}
 
 	void set_feed(std::shared_ptr<RssFeed> fd);
@@ -100,27 +98,25 @@ private:
 	void handle_cmdline_num(unsigned int idx);
 
 	std::string gen_flags(std::shared_ptr<RssItem> item);
-	std::string gen_datestr(time_t t, const std::string& datetimeformat);
 
 	void prepare_set_filterpos();
 
-	void invalidate(InvalidationMode m)
+	void invalidate_everything()
 	{
-		assert(m == InvalidationMode::COMPLETE);
-
 		invalidated = true;
 		invalidation_mode = InvalidationMode::COMPLETE;
 	}
 
-	void invalidate(const unsigned int pos)
+	void invalidate(const unsigned int invalidated_pos)
 	{
 		if (invalidated == true &&
-			invalidation_mode == InvalidationMode::COMPLETE)
+			invalidation_mode == InvalidationMode::COMPLETE) {
 			return;
+		}
 
 		invalidated = true;
 		invalidation_mode = InvalidationMode::PARTIAL;
-		invalidated_itempos.push_back(pos);
+		invalidated_itempos.push_back(invalidated_pos);
 	}
 
 	std::string item2formatted_line(const ItemPtrPosPair& item,
@@ -131,7 +127,7 @@ private:
 	unsigned int pos;
 	std::shared_ptr<RssFeed> feed;
 	bool apply_filter;
-	Matcher m;
+	Matcher matcher;
 	std::vector<ItemPtrPosPair> visible_items;
 	bool show_searchresult;
 	std::string searchphrase;
@@ -158,6 +154,8 @@ private:
 	ListFormatter listfmt;
 	Cache* rsscache;
 	FilterContainer* filters;
+
+	void handle_op_saveall();
 };
 
 } // namespace newsboat
