@@ -13,7 +13,7 @@ protected:
 	void process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
-	void open_unread_items_in_browser(std::shared_ptr<RssFeed> feed,
+	int open_unread_items_in_browser(std::shared_ptr<RssFeed> feed,
 		bool markread);
 };
 
