@@ -5,7 +5,7 @@
 #include "strprintf.h"
 
 namespace newsboat {
-	enum class Level;
+enum class Level;
 }
 
 extern "C" {
@@ -19,17 +19,18 @@ namespace newsboat {
 enum class Level { NONE = 0, USERERROR, CRITICAL, ERROR, WARN, INFO, DEBUG };
 
 namespace Logger {
-	void set_logfile(const std::string& logfile);
-	void set_user_error_logfile(const std::string& logfile);
-	void set_loglevel(Level l);
+void set_logfile(const std::string& logfile);
+void set_user_error_logfile(const std::string& logfile);
+void set_loglevel(Level l);
 
-	template<typename... Args>
-	void log(Level l, const std::string& format, Args... args)
-	{
-		if (static_cast<uint64_t>(l) <= rs_get_loglevel()) {
-			rs_log(l, strprintf::fmt(format, args...).c_str());
-		}
+template<typename... Args>
+void log(Level l, const std::string& format, Args... args)
+{
+	if (l == Level::USERERROR
+		|| static_cast<uint64_t>(l) <= rs_get_loglevel()) {
+		rs_log(l, strprintf::fmt(format, args...).c_str());
 	}
+}
 };
 
 } // namespace newsboat
