@@ -4,8 +4,7 @@
 #include <json.h>
 
 #include "remoteapi.h"
-#include "rsspp.h"
-#include "urlreader.h"
+#include "rss/feed.h"
 
 #define ID_SEPARATOR "/////"
 
@@ -32,25 +31,12 @@ private:
 	json_object* query_api(const std::string& url,
 		const std::string* postdata);
 	std::map<std::string, std::vector<std::string>> mk_feeds_to_tags(
-		json_object*);
+			json_object*);
 	std::string api_location;
 	FeedMap known_feeds;
 	unsigned int min_pages;
 };
 
-class NewsBlurUrlReader : public UrlReader {
-public:
-	NewsBlurUrlReader(const std::string& url_file, RemoteApi* a);
-	~NewsBlurUrlReader() override;
-	void write_config() override;
-	void reload() override;
-	std::string get_source() override;
-
-private:
-	std::string file;
-	RemoteApi* api;
-};
-
 } // namespace newsboat
 
 #endif /* NEWSBOAT_NEWSBLURAPI_H_ */
