@@ -5,8 +5,7 @@
 #include <map>
 
 #include "remoteapi.h"
-#include "rsspp.h"
-#include "urlreader.h"
+#include "rss/feed.h"
 
 namespace newsboat {
 
@@ -36,19 +35,6 @@ private:
 	FeedMap known_feeds;
 };
 
-class OcNewsUrlReader : public UrlReader {
-public:
-	OcNewsUrlReader(const std::string& url_file, RemoteApi* a);
-	~OcNewsUrlReader() override;
-	void write_config() override;
-	void reload() override;
-	std::string get_source() override;
-
-private:
-	std::string file;
-	RemoteApi* api;
-};
-
 } // namespace newsboat
 
 #endif /* NEWSBOAT_OCNEWSAPI_H_ */
