@@ -4,20 +4,20 @@
 #include <vector>
 
 #include "download.h"
-#include "pb_controller.h"
+#include "pbcontroller.h"
 
 namespace podboat {
 
-class queueloader {
+class QueueLoader {
 public:
-	queueloader(const std::string& file, pb_controller* c = 0);
-	void reload(std::vector<download>& downloads,
+	QueueLoader(const std::string& file, PbController* c = 0);
+	void reload(std::vector<Download>& downloads,
 		bool remove_unplayed = false);
 
 private:
 	std::string get_filename(const std::string& str);
 	std::string queuefile;
-	pb_controller* ctrl;
+	PbController* ctrl;
 };
 
 } // namespace podboat
