@@ -4,20 +4,25 @@
 #include <mutex>
 #include <vector>
 
+#include "configcontainer.h"
+
 namespace newsboat {
 
-class controller;
-class curl_handle;
+class Cache;
+class Controller;
+class CurlHandle;
 
-/// \brief Updates feeds (fetches, parses, puts results into controller).
+/// \brief Updates feeds (fetches, parses, puts results into Controller).
 class Reloader {
-	controller* ctrl;
+	Controller* ctrl;
+	Cache* rsscache;
+	ConfigContainer* cfg;
 	std::mutex reload_mutex;
 
 	std::string prepare_message(unsigned int pos, unsigned int max);
 
 public:
-	Reloader(controller* c);
+	Reloader(Controller* c, Cache* cc, ConfigContainer* cfg);
 
 	/// \brief Creates detached thread that runs periodic updates.
 	void spawn_reloadthread();
@@ -49,7 +54,7 @@ public:
 	void reload(unsigned int pos,
 		unsigned int max = 0,
 		bool unattended = false,
-		curl_handle* easyhandle = nullptr);
+		CurlHandle* easyhandle = nullptr);
 
 	/// \brief Reloads all feeds, spawning threads as necessary.
 	///
@@ -83,6 +88,7 @@ public:
 	/// If "notify-screen", "notify-xterm" or "notify-program" is chosen, the
 	/// notification will contain \a msg passed.
 	void notify(const std::string& msg);
+
 };
 
 } // namespace newsboat
