@@ -10,7 +10,7 @@ use std::io::Write;
 use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::Mutex;
 
-#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
+#[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Debug)]
 /// "Importance levels" for log messages.
 ///
 /// Each message is assigned a level. set_loglevel instructs Logger to only store messages at and
@@ -71,6 +71,12 @@ impl fmt::Display for Level {
     }
 }
 
+impl Default for Level {
+    fn default() -> Level {
+        Level::None
+    }
+}
+
 /// Stores the handles for logfiles.
 ///
 /// This is part of `Logger` struct. This struct is not thread-safe, but in `Logger`, it will be
@@ -193,11 +199,9 @@ impl Logger {
     ///
     /// This method is a wrapper around `log_raw()`.
     pub fn log(&self, level: Level, message: &str) {
-        if level as usize > self.get_loglevel() {
-            return;
+        if level == Level::UserError || level as usize <= self.get_loglevel() {
+            self.log_raw(level, message.as_bytes())
         }
-
-        self.log_raw(level, message.as_bytes())
     }
 
     /// Writes binary data to the log.
@@ -232,14 +236,16 @@ impl Logger {
 
         let mut files = self.files.lock().expect("Someone poisoned logger's mutex");
 
-        if let Some(ref mut logfile) = files.logfile {
-            let level = format!("{}: ", level);
+        if level as usize <= self.get_loglevel() {
+            if let Some(ref mut logfile) = files.logfile {
+                let level = format!("{}: ", level);
 
-            // Ignoring the error since checking every log() call will be too bothersome.
-            let _ = logfile.write_all(timestamp.as_bytes());
-            let _ = logfile.write_all(level.as_bytes());
-            let _ = logfile.write_all(data);
-            let _ = logfile.write_all("\n".as_bytes());
+                // Ignoring the error since checking every log() call will be too bothersome.
+                let _ = logfile.write_all(timestamp.as_bytes());
+                let _ = logfile.write_all(level.as_bytes());
+                let _ = logfile.write_all(data);
+                let _ = logfile.write_all("\n".as_bytes());
+            }
         }
 
         if level == Level::UserError {
@@ -280,7 +286,7 @@ static GLOBAL_LOGGER: OnceCell<Logger> = OnceCell::INIT;
 /// This logger exists for the duration of the program. It's better to set the loglevel and
 /// logfiles as early as possible, so no messages are lost.
 pub fn get_instance() -> &'static Logger {
-    GLOBAL_LOGGER.get_or_init(|| Logger::new())
+    GLOBAL_LOGGER.get_or_init(Logger::new)
 }
 
 /// Convenience macro for logging.
@@ -737,17 +743,11 @@ mod tests {
     }
 
     #[test]
-    fn t_writes_to_errorlog_at_usererror_and_above() {
+    fn t_writes_to_errorlog_at_all_loglevels() {
         let message = (Level::UserError, "hello".to_string());
 
-        LogLinesCounter::new()
-            .with_messages(vec![message.clone()])
-            .at_levels(vec![Level::None])
-            .expected_errorlog_lines_count(0)
-            .test()
-            .unwrap();
-
         let log_levels = vec![
+            Level::None,
             Level::UserError,
             Level::Critical,
             Level::Error,
@@ -773,15 +773,8 @@ mod tests {
             ),
         ];
 
-        let nolog_levels = vec![Level::None];
-        LogLinesCounter::new()
-            .with_messages(messages.clone())
-            .at_levels(nolog_levels)
-            .expected_errorlog_lines_count(0)
-            .test()
-            .unwrap();
-
         let log_levels = vec![
+            Level::None,
             Level::UserError,
             Level::Critical,
             Level::Error,
