@@ -1,15 +1,25 @@
+extern crate curl_sys;
 extern crate dirs;
+extern crate libc;
+extern crate natord;
 extern crate rand;
 extern crate regex;
+extern crate std;
+extern crate unicode_segmentation;
 extern crate unicode_width;
 extern crate url;
 
 use self::regex::Regex;
+use self::unicode_segmentation::UnicodeSegmentation;
 use self::unicode_width::UnicodeWidthStr;
 use self::url::percent_encoding::*;
 use self::url::Url;
+use libc::c_ulong;
 use logger::{self, Level};
-use std::io::Write;
+use std::fs::DirBuilder;
+use std::io::{self, Write};
+use std::os::unix::fs::DirBuilderExt;
+use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
 
 pub fn replace_all(input: String, from: &str, to: &str) -> String {
@@ -71,7 +81,7 @@ pub fn absolute_url(base_url: &str, link: &str) -> String {
     Url::parse(base_url)
         .and_then(|url| url.join(link))
         .as_ref()
-        .map(|url| url.as_str())
+        .map(Url::as_str)
         .unwrap_or(link)
         .to_owned()
 }
@@ -97,7 +107,16 @@ pub fn resolve_tilde(path: String) -> String {
             }
         }
     }
-    return file_path;
+    file_path
+}
+
+pub fn resolve_relative(reference: &Path, path: &Path) -> PathBuf {
+    if path.is_relative() {
+        // Will only ever panic if reference is `/`, which shouldn't be the case as reference is
+        // always a file path
+        return reference.parent().unwrap().join(path);
+    }
+    path.to_path_buf()
 }
 
 pub fn is_special_url(url: &str) -> bool {
@@ -176,7 +195,7 @@ pub fn censor_url(url: &str) -> String {
                 url
             })
             .as_ref()
-            .map(|url| url.as_str())
+            .map(Url::as_str)
             .unwrap_or(url)
             .to_owned()
     } else {
@@ -184,12 +203,39 @@ pub fn censor_url(url: &str) -> String {
     }
 }
 
+/// Quote a string for use with stfl by replacing all occurences of "<" with "<>"
+/// ```
+/// use libnewsboat::utils::quote_for_stfl;
+/// assert_eq!(&quote_for_stfl("<"), "<>");
+/// assert_eq!(&quote_for_stfl("<<><><><"), "<><>><>><>><>");
+/// assert_eq!(&quote_for_stfl("test"), "test");
+/// ```
+pub fn quote_for_stfl(string: &str) -> String {
+    return string.replace("<", "<>");
+}
+
+/// Get basename from a URL if available else return an empty string
+/// ```
+/// use libnewsboat::utils::get_basename;
+/// assert_eq!(get_basename("https://example.com/"), "");
+/// assert_eq!(get_basename("https://example.org/?param=value#fragment"), "");
+/// assert_eq!(get_basename("https://example.org/path/to/?param=value#fragment"), "");
+/// assert_eq!(get_basename("https://example.org/file.mp3"), "file.mp3");
+/// assert_eq!(get_basename("https://example.org/path/to/file.mp3?param=value#fragment"), "file.mp3");
+/// ```
+pub fn get_basename(input: &str) -> String {
+    match Url::parse(input) {
+        Ok(url) => match url.path_segments() {
+            Some(segments) => segments.last().unwrap().to_string(),
+            None => String::from(""),
+        },
+        Err(_) => String::from(""),
+    }
+}
+
 pub fn get_default_browser() -> String {
     use std::env;
-    match env::var("BROWSER") {
-        Ok(val) => return val,
-        Err(_) => return String::from("lynx"),
-    }
+    env::var("BROWSER").unwrap_or_else(|_| "lynx".to_string())
 }
 
 pub fn trim(rs_str: String) -> String {
@@ -209,10 +255,9 @@ pub fn quote(input: String) -> String {
 }
 
 pub fn quote_if_necessary(input: String) -> String {
-    if input.find(" ") != None {
-        quote(input)
-    } else {
-        input
+    match input.find(' ') {
+        Some(_) => quote(input),
+        None => input,
     }
 }
 
@@ -226,19 +271,15 @@ pub fn is_valid_color(color: &str) -> bool {
     ];
 
     if COLORS.contains(&color) {
-        return true;
-    }
-    if color.starts_with("color0") {
-        return color == "color0";
-    }
-    if color.starts_with("color") {
+        true
+    } else if color.starts_with("color0") {
+        color == "color0"
+    } else if color.starts_with("color") {
         let num_part = &color[5..];
-        match num_part.parse::<u8>() {
-            Ok(_) => return true,
-            _ => return false,
-        }
+        num_part.parse::<u8>().is_ok()
+    } else {
+        false
     }
-    false
 }
 
 pub fn is_valid_attribute(attribute: &str) -> bool {
@@ -257,29 +298,15 @@ pub fn is_valid_attribute(attribute: &str) -> bool {
 }
 
 pub fn strwidth(rs_str: &str) -> usize {
-    let control = rs_str.chars().fold(true, |acc, x| acc & !x.is_control());
-
-    if control {
-        return UnicodeWidthStr::width(rs_str);
-    } else {
-        return rs_str.len();
-    }
+    UnicodeWidthStr::width(rs_str)
 }
 
 pub fn strwidth_stfl(rs_str: &str) -> usize {
-    let mut reduce: usize = 0;
-    let mut chars = rs_str.chars().peekable();
-    loop {
-        match chars.next() {
-            Some('<') => {
-                if chars.peek() != Some(&'>') {
-                    reduce += 3;
-                }
-            }
-            Some(_) => continue,
-            None => break,
-        }
-    }
+    let reduce = 3 * rs_str
+        .chars()
+        .zip(rs_str.chars().skip(1))
+        .filter(|&(c, next_c)| c == '<' && next_c != '>')
+        .count();
 
     let width = strwidth(rs_str);
     if width < reduce {
@@ -289,6 +316,13 @@ pub fn strwidth_stfl(rs_str: &str) -> usize {
     }
 }
 
+/// Remove all soft-hyphens as they can behave unpredictably (see
+/// https://github.com/akrennmair/newsbeuter/issues/259#issuecomment-259609490) and inadvertently
+/// render as hyphens
+pub fn remove_soft_hyphens(text: &mut String) {
+    text.retain(|c| c != '\u{00AD}')
+}
+
 pub fn is_valid_podcast_type(mimetype: &str) -> bool {
     let re = Regex::new(r"(audio|video)/.*").unwrap();
     let matches = re.is_match(mimetype);
@@ -299,20 +333,41 @@ pub fn is_valid_podcast_type(mimetype: &str) -> bool {
     matches || found
 }
 
-pub fn unescape_url(rs_str: String) -> Option<String> {
-    let result = percent_decode(rs_str.as_bytes());
-    let result = result.decode_utf8();
-    if result.is_err() {
-        return None;
+pub fn get_auth_method(method: &str) -> c_ulong {
+    match method {
+        "basic" => curl_sys::CURLAUTH_BASIC,
+        "digest" => curl_sys::CURLAUTH_DIGEST,
+        "digest_ie" => curl_sys::CURLAUTH_DIGEST_IE,
+        "gssnegotiate" => curl_sys::CURLAUTH_GSSNEGOTIATE,
+        "ntlm" => curl_sys::CURLAUTH_NTLM,
+        "anysafe" => curl_sys::CURLAUTH_ANYSAFE,
+        "any" | "" => curl_sys::CURLAUTH_ANY,
+        _ => {
+            log!(
+                Level::UserError,
+                "utils::get_auth_method: you configured an invalid proxy authentication method: {}",
+                method
+            );
+            curl_sys::CURLAUTH_ANY
+        }
     }
+}
 
-    Some(result.unwrap().replace("\0", ""))
+pub fn unescape_url(rs_str: String) -> Option<String> {
+    let decoded = percent_decode(rs_str.as_bytes()).decode_utf8();
+    decoded.ok().map(|s| s.replace("\0", ""))
 }
 
 /// Runs given command in a shell, and returns the output (from stdout; stderr is printed to the
 /// screen).
 pub fn get_command_output(cmd: &str) -> String {
-    let cmd = Command::new("sh").arg("-c").arg(cmd).output();
+    let cmd = Command::new("sh")
+        .arg("-c")
+        .arg(cmd)
+        // Inherit stdin so that the program can ask something of the user (see
+        // https://github.com/newsboat/newsboat/issues/455 for an example).
+        .stdin(Stdio::inherit())
+        .output();
     // from_utf8_lossy will convert any bad bytes to U+FFFD
     cmd.map(|cmd| String::from_utf8_lossy(&cmd.stdout).into_owned())
         .unwrap_or_else(|_| String::from(""))
@@ -408,8 +463,7 @@ pub fn make_title(rs_str: String) -> String {
     // Throw away common webpage suffixes: .html, .php, .aspx, .htm
     result = result
         .trim_right_matches(".html")
-        .trim_right_matches(".php");
-    result = result
+        .trim_right_matches(".php")
         .trim_right_matches(".aspx")
         .trim_right_matches(".htm");
 
@@ -432,6 +486,153 @@ pub fn make_title(rs_str: String) -> String {
     }
 }
 
+/// Get the current working directory.
+pub fn getcwd() -> Result<PathBuf, io::Error> {
+    use std::env;
+    env::current_dir()
+}
+
+pub fn strnaturalcmp(a: &str, b: &str) -> std::cmp::Ordering {
+    natord::compare(a, b)
+}
+
+/// Calculate the number of padding tabs when formatting columns
+///
+/// The number of tabs will be adjusted by the width of the given string.  Usually, a column will
+/// consist of 4 tabs, 8 characters each.  Each column will consist of at least one tab.
+///
+/// ```
+/// use libnewsboat::utils::gentabs;
+///
+/// fn genstring(len: usize) -> String {
+///     return std::iter::repeat("a").take(len).collect::<String>();
+/// }
+///
+/// assert_eq!(gentabs(""), 4);
+/// assert_eq!(gentabs("a"), 4);
+/// assert_eq!(gentabs("aa"), 4);
+/// assert_eq!(gentabs("aaa"), 4);
+/// assert_eq!(gentabs("aaaa"), 4);
+/// assert_eq!(gentabs("aaaaa"), 4);
+/// assert_eq!(gentabs("aaaaaa"), 4);
+/// assert_eq!(gentabs("aaaaaaa"), 4);
+/// assert_eq!(gentabs("aaaaaaaa"), 3);
+/// assert_eq!(gentabs(&genstring(8)), 3);
+/// assert_eq!(gentabs(&genstring(9)), 3);
+/// assert_eq!(gentabs(&genstring(15)), 3);
+/// assert_eq!(gentabs(&genstring(16)), 2);
+/// assert_eq!(gentabs(&genstring(20)), 2);
+/// assert_eq!(gentabs(&genstring(24)), 1);
+/// assert_eq!(gentabs(&genstring(32)), 1);
+/// assert_eq!(gentabs(&genstring(100)), 1);
+/// ```
+pub fn gentabs(string: &str) -> usize {
+    let tabcount = strwidth(string) / 8;
+    if tabcount >= 4 {
+        1
+    } else {
+        4 - tabcount
+    }
+}
+
+/// Recursively create directories if missing and set permissions accordingly.
+pub fn mkdir_parents<R: AsRef<Path>>(p: &R, mode: u32) -> io::Result<()> {
+    DirBuilder::new()
+        .mode(mode)
+        .recursive(true) // directories created with same security and permissions
+        .create(p.as_ref())
+}
+
+/// Counts graphemes in a given string.
+///
+/// ```
+/// use libnewsboat::utils::graphemes_count;
+///
+/// assert_eq!(graphemes_count("D"), 1);
+/// // len() counts bytes, not characters, but all ASCII symbols are represented by one byte in
+/// // UTF-8, so len() returns 1 in this case
+/// assert_eq!("D".len(), 1);
+///
+/// // Here's a situation where a single grapheme is represented by multiple bytes
+/// assert_eq!(graphemes_count("Ж"), 1);
+/// assert_eq!("Ж".len(), 2);
+///
+/// assert_eq!(graphemes_count("📰"), 1);
+/// assert_eq!("📰".len(), 4);
+/// ```
+pub fn graphemes_count(input: &str) -> usize {
+    UnicodeSegmentation::graphemes(input, true).count()
+}
+
+/// Extracts up to `n` first graphemes from the given string.
+///
+/// ```
+/// use libnewsboat::utils::take_graphemes;
+///
+/// let input = "Привет!";
+/// assert_eq!(take_graphemes(input, 1), "П");
+/// assert_eq!(take_graphemes(input, 4), "Прив");
+/// assert_eq!(take_graphemes(input, 6), "Привет");
+/// assert_eq!(take_graphemes(input, 20), input);
+/// ```
+pub fn take_graphemes(input: &str, n: usize) -> String {
+    UnicodeSegmentation::graphemes(input, true)
+        .take(n)
+        .collect::<String>()
+}
+
+/// The tag and Git commit ID the program was built from, or a pre-defined value from config.h if
+/// there is no Git directory.
+pub fn program_version() -> String {
+    // NEWSBOAT_VERSION is set by this crate's build script, "build.rs"
+    env!("NEWSBOAT_VERSION").to_string()
+}
+
+/// Newsboat's major version number.
+pub fn newsboat_major_version() -> u32 {
+    // This will panic if the version couldn't be parsed, which is virtually impossible as Cargo
+    // won't even start compilation if it couldn't parse the version.
+    env!("CARGO_PKG_VERSION_MAJOR").parse::<u32>().unwrap()
+}
+
+/// Returns the part of the string before first # character (or the whole input string if there are
+/// no # character in it). Pound characters inside double quotes and backticks are ignored.
+pub fn strip_comments(line: &str) -> &str {
+    let mut prev_was_backslash = false;
+    let mut inside_quotes = false;
+    let mut inside_backticks = false;
+
+    let mut first_pound_chr_idx = line.len();
+
+    for (idx, chr) in line.char_indices() {
+        if chr == '\\' {
+            prev_was_backslash = true;
+            continue;
+        } else if chr == '"' {
+            // If the quote is escaped or we're inside backticks, do nothing
+            if !prev_was_backslash && !inside_backticks {
+                inside_quotes = !inside_quotes;
+            }
+        } else if chr == '`' {
+            // If the backtick is escaped, do nothing
+            if !prev_was_backslash {
+                inside_backticks = !inside_backticks;
+            }
+        } else if chr == '#' {
+            if !prev_was_backslash && !inside_quotes && !inside_backticks {
+                first_pound_chr_idx = idx;
+                break;
+            }
+        }
+
+        // We call `continue` when we run into a backslash; here, we handle all the other
+        // characters, which clearly *aren't* a backslash
+        prev_was_backslash = false;
+    }
+
+    &line[0..first_pound_chr_idx]
+}
+
 #[cfg(test)]
 mod tests {
     extern crate tempfile;
@@ -622,20 +823,20 @@ mod tests {
 
     #[test]
     fn t_strwidth() {
-        assert!(strwidth("") == 0);
-        assert!(strwidth("xx") == 2);
-        assert!(strwidth("\u{F91F}") == 2);
-        assert!(strwidth("\u{0007}") == 1);
+        assert_eq!(strwidth(""), 0);
+        assert_eq!(strwidth("xx"), 2);
+        assert_eq!(strwidth("\u{F91F}"), 2);
+        assert_eq!(strwidth("\u{0007}"), 0);
     }
 
     #[test]
     fn t_strwidth_stfl() {
-        assert!(strwidth_stfl("") == 0);
-        assert!(strwidth_stfl("x<hi>x") == 3);
-        assert!(strwidth_stfl("x<>x") == 4);
-        assert!(strwidth_stfl("\u{F91F}") == 2);
-        assert!(strwidth_stfl("\u{0007}") == 1);
-        assert!(strwidth_stfl("<a") == 0); // #415
+        assert_eq!(strwidth_stfl(""), 0);
+        assert_eq!(strwidth_stfl("x<hi>x"), 3);
+        assert_eq!(strwidth_stfl("x<>x"), 4);
+        assert_eq!(strwidth_stfl("\u{F91F}"), 2);
+        assert_eq!(strwidth_stfl("\u{0007}"), 0);
+        assert_eq!(strwidth_stfl("<a"), 0); // #415
     }
 
     #[test]
@@ -675,6 +876,23 @@ mod tests {
         }
     }
 
+    #[test]
+    fn t_get_auth_method() {
+        assert_eq!(get_auth_method("any"), curl_sys::CURLAUTH_ANY);
+        assert_eq!(get_auth_method("ntlm"), curl_sys::CURLAUTH_NTLM);
+        assert_eq!(get_auth_method("basic"), curl_sys::CURLAUTH_BASIC);
+        assert_eq!(get_auth_method("digest"), curl_sys::CURLAUTH_DIGEST);
+        assert_eq!(get_auth_method("digest_ie"), curl_sys::CURLAUTH_DIGEST_IE);
+        assert_eq!(
+            get_auth_method("gssnegotiate"),
+            curl_sys::CURLAUTH_GSSNEGOTIATE
+        );
+        assert_eq!(get_auth_method("anysafe"), curl_sys::CURLAUTH_ANYSAFE);
+
+        assert_eq!(get_auth_method(""), curl_sys::CURLAUTH_ANY);
+        assert_eq!(get_auth_method("unknown"), curl_sys::CURLAUTH_ANY);
+    }
+
     #[test]
     fn t_unescape_url() {
         assert!(unescape_url(String::from("foo%20bar")).unwrap() == String::from("foo bar"));
@@ -697,6 +915,7 @@ mod tests {
             get_command_output("a-program-that-is-guaranteed-to-not-exists"),
             "".to_string()
         );
+        assert_eq!(get_command_output("echo c\" d e"), "".to_string());
     }
 
     #[test]
@@ -714,7 +933,17 @@ mod tests {
 
         run_command("touch", filepath.to_str().unwrap());
 
-        thread::sleep(time::Duration::from_millis(10));
+        // Busy-wait for 10 tries of 10 milliseconds each, waiting for `touch` to
+        // create the file. Usually it happens quickly, and the loop exists on the
+        // first try; but sometimes on CI it takes longer for `touch` to finish, so
+        // we need a slightly longer wait.
+        for _ in 0..10 {
+            thread::sleep(time::Duration::from_millis(10));
+
+            if filepath.exists() {
+                break;
+            }
+        }
 
         assert!(filepath.exists());
     }
@@ -747,56 +976,220 @@ mod tests {
     #[test]
     fn t_make_title() {
         let mut input = String::from("http://example.com/Item");
-        assert!(make_title(input) == String::from("Item"));
+        assert_eq!(make_title(input), String::from("Item"));
 
         input = String::from("http://example.com/This-is-the-title");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/This_is_the_title");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/This_is-the_title");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/This_is-the_title.php");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/This_is-the_title.html");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/This_is-the_title.htm");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/This_is-the_title.aspx");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/this-is-the-title");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/items/misc/this-is-the-title");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/item/");
-        assert!(make_title(input) == String::from("Item"));
+        assert_eq!(make_title(input), String::from("Item"));
 
         input = String::from("http://example.com/item/////////////");
-        assert!(make_title(input) == String::from("Item"));
+        assert_eq!(make_title(input), String::from("Item"));
 
         input = String::from("blahscheme://example.com/this-is-the-title");
-        assert!(make_title(input) == String::from("This is the title"));
+        assert_eq!(make_title(input), String::from("This is the title"));
 
         input = String::from("http://example.com/story/aug/title-with-dashes?a=b");
-        assert!(make_title(input) == String::from("Title with dashes"));
+        assert_eq!(make_title(input), String::from("Title with dashes"));
 
         input = String::from("http://example.com/title-with-dashes?a=b&x=y&utf8=✓");
-        assert!(make_title(input) == String::from("Title with dashes"));
+        assert_eq!(make_title(input), String::from("Title with dashes"));
 
         input = String::from("https://example.com/It%27s%202017%21");
-        assert!(make_title(input) == String::from("It's 2017!"));
+        assert_eq!(make_title(input), String::from("It's 2017!"));
 
         input = String::from("https://example.com/?format=rss");
-        assert!(make_title(input) == String::from(""));
+        assert_eq!(make_title(input), String::from(""));
 
-        assert!(make_title(String::from("")) == String::from(""));
+        assert_eq!(make_title(String::from("")), String::from(""));
+    }
+
+    #[test]
+    fn t_resolve_relative() {
+        assert_eq!(
+            resolve_relative(Path::new("/foo/bar"), Path::new("/baz")),
+            Path::new("/baz")
+        );
+        assert_eq!(
+            resolve_relative(Path::new("/config"), Path::new("/config/baz")),
+            Path::new("/config/baz")
+        );
+
+        assert_eq!(
+            resolve_relative(Path::new("/foo/bar"), Path::new("baz")),
+            Path::new("/foo/baz")
+        );
+        assert_eq!(
+            resolve_relative(Path::new("/config"), Path::new("baz")),
+            Path::new("/baz")
+        );
+    }
+
+    #[test]
+    fn t_remove_soft_hyphens_removes_all_00ad_unicode_chars_from_a_string() {
+        {
+            // Does nothing if input has no soft hyphens in it
+            let mut input1 = "hello world!".to_string();
+            remove_soft_hyphens(&mut input1);
+            assert_eq!(input1, "hello world!");
+        }
+
+        {
+            // Removes *all* soft hyphens
+            let mut data = "hy\u{00AD}phen\u{00AD}a\u{00AD}tion".to_string();
+            remove_soft_hyphens(&mut data);
+            assert_eq!(data, "hyphenation");
+        }
+
+        {
+            // Removes consecutive soft hyphens
+            let mut data = "don't know why any\u{00AD}\u{00AD}one would do that".to_string();
+            remove_soft_hyphens(&mut data);
+            assert_eq!(data, "don't know why anyone would do that");
+        }
+
+        {
+            // Removes soft hyphen at the beginning of the line
+            let mut data = "\u{00AD}tion".to_string();
+            remove_soft_hyphens(&mut data);
+            assert_eq!(data, "tion");
+        }
+
+        {
+            // Removes soft hyphen at the end of the line
+            let mut data = "over\u{00AD}".to_string();
+            remove_soft_hyphens(&mut data);
+            assert_eq!(data, "over");
+        }
+    }
+
+    #[test]
+    fn t_mkdir_parents() {
+        use self::tempfile::TempDir;
+        use std::fs;
+        use std::os::unix::fs::PermissionsExt;
+
+        let mode: u32 = 0o700;
+        let tmp_dir = TempDir::new().unwrap();
+        let path = tmp_dir.path().join("parent/dir");
+        assert_eq!(path.exists(), false);
+
+        let result = mkdir_parents(&path, mode);
+        assert!(result.is_ok());
+        assert_eq!(path.exists(), true);
+
+        let file_type_mask = 0o7777;
+        let metadata = fs::metadata(&path).unwrap();
+        assert_eq!(file_type_mask & metadata.permissions().mode(), mode);
+
+        // rerun on existing directories
+        let result = mkdir_parents(&path, mode);
+        assert!(result.is_ok());
+    }
+
+    #[test]
+    fn t_strnaturalcmp() {
+        use std::cmp::Ordering;
+        assert_eq!(strnaturalcmp("", ""), Ordering::Equal);
+        assert_eq!(strnaturalcmp("", "a"), Ordering::Less);
+        assert_eq!(strnaturalcmp("a", ""), Ordering::Greater);
+        assert_eq!(strnaturalcmp("a", "a"), Ordering::Equal);
+        assert_eq!(strnaturalcmp("", "9"), Ordering::Less);
+        assert_eq!(strnaturalcmp("9", ""), Ordering::Greater);
+        assert_eq!(strnaturalcmp("1", "1"), Ordering::Equal);
+        assert_eq!(strnaturalcmp("1", "2"), Ordering::Less);
+        assert_eq!(strnaturalcmp("3", "2"), Ordering::Greater);
+        assert_eq!(strnaturalcmp("a1", "a1"), Ordering::Equal);
+        assert_eq!(strnaturalcmp("a1", "a2"), Ordering::Less);
+        assert_eq!(strnaturalcmp("a2", "a1"), Ordering::Greater);
+        assert_eq!(strnaturalcmp("a1a2", "a1a3"), Ordering::Less);
+        assert_eq!(strnaturalcmp("a1a2", "a1a0"), Ordering::Greater);
+        assert_eq!(strnaturalcmp("134", "122"), Ordering::Greater);
+        assert_eq!(strnaturalcmp("12a3", "12a3"), Ordering::Equal);
+        assert_eq!(strnaturalcmp("12a1", "12a0"), Ordering::Greater);
+        assert_eq!(strnaturalcmp("12a1", "12a2"), Ordering::Less);
+        assert_eq!(strnaturalcmp("a", "aa"), Ordering::Less);
+        assert_eq!(strnaturalcmp("aaa", "aa"), Ordering::Greater);
+        assert_eq!(strnaturalcmp("Alpha 2", "Alpha 2"), Ordering::Equal);
+        assert_eq!(strnaturalcmp("Alpha 2", "Alpha 2A"), Ordering::Less);
+        assert_eq!(strnaturalcmp("Alpha 2 B", "Alpha 2"), Ordering::Greater);
+
+        assert_eq!(strnaturalcmp("aa10", "aa2"), Ordering::Greater);
+    }
+
+    #[test]
+    fn t_strip_comments() {
+        // no comments in line
+        assert_eq!(strip_comments(""), "");
+        assert_eq!(strip_comments("\t\n"), "\t\n");
+        assert_eq!(strip_comments("some directive "), "some directive ");
+
+        // fully commented line
+        assert_eq!(strip_comments("#"), "");
+        assert_eq!(strip_comments("# #"), "");
+        assert_eq!(strip_comments("# comment"), "");
+
+        // partially commented line
+        assert_eq!(strip_comments("directive # comment"), "directive ");
+        assert_eq!(
+            strip_comments("directive # comment # another"),
+            "directive "
+        );
+        assert_eq!(strip_comments("directive#comment"), "directive");
+
+        // ignores # characters inside double quotes (#652)
+        let expected = r#"highlight article "[-=+#_*~]{3,}.*" green default"#;
+        let input = expected.to_owned() + "# this is a comment";
+        assert_eq!(strip_comments(&input), expected);
+
+        let expected =
+            r#"highlight all "(https?|ftp)://[\-\.,/%~_:?&=\#a-zA-Z0-9]+" blue default bold"#;
+        let input = expected.to_owned() + "#heresacomment";
+        assert_eq!(strip_comments(&input), expected);
+
+        // Escaped double quote inside double quotes is not treated as closing quote
+        let expected = r#"test "here \"goes # nothing\" etc" hehe"#;
+        let input = expected.to_owned() + "# and here is a comment";
+        assert_eq!(strip_comments(&input), expected);
+
+        // Ignores # characters inside backticks
+        let expected = r#"one `two # three` four"#;
+        let input = expected.to_owned() + "# and a comment, of course";
+        assert_eq!(strip_comments(&input), expected);
+
+        // Escaped backtick inside backticks is not treated as closing
+        let expected = r#"some `other \` tricky # test` hehe"#;
+        let input = expected.to_owned() + "#here goescomment";
+        assert_eq!(strip_comments(&input), expected);
+
+        // Ignores escaped # characters (\\#)
+        let expected = r#"one two \# three four"#;
+        let input = expected.to_owned() + "# and a comment";
+        assert_eq!(strip_comments(&input), expected);
     }
 }
