@@ -1,13 +1,14 @@
 #include "colormanager.h"
 
 #include "config.h"
-#include "exceptions.h"
+#include "confighandlerexception.h"
 #include "feedlistformaction.h"
 #include "filebrowserformaction.h"
 #include "helpformaction.h"
 #include "itemlistformaction.h"
 #include "itemviewformaction.h"
 #include "logger.h"
+#include "matcherexception.h"
 #include "pbview.h"
 #include "selectformaction.h"
 #include "strprintf.h"
@@ -52,17 +53,17 @@ void ColorManager::handle_action(const std::string& action,
 
 		if (!utils::is_valid_color(fgcolor))
 			throw ConfigHandlerException(strprintf::fmt(
-				_("`%s' is not a valid color"), fgcolor));
+					_("`%s' is not a valid color"), fgcolor));
 		if (!utils::is_valid_color(bgcolor))
 			throw ConfigHandlerException(strprintf::fmt(
-				_("`%s' is not a valid color"), bgcolor));
+					_("`%s' is not a valid color"), bgcolor));
 
 		std::vector<std::string> attribs;
 		for (unsigned int i = 3; i < params.size(); ++i) {
 			if (!utils::is_valid_attribute(params[i]))
 				throw ConfigHandlerException(strprintf::fmt(
-					_("`%s' is not a valid attribute"),
-					params[i]));
+						_("`%s' is not a valid attribute"),
+						params[i]));
 			attribs.push_back(params[i]);
 		}
 
@@ -78,8 +79,8 @@ void ColorManager::handle_action(const std::string& action,
 			colors_loaded_ = true;
 		} else
 			throw ConfigHandlerException(strprintf::fmt(
-				_("`%s' is not a valid configuration element"),
-				element));
+					_("`%s' is not a valid configuration element"),
+					element));
 
 	} else
 		throw ConfigHandlerException(
@@ -90,9 +91,9 @@ void ColorManager::dump_config(std::vector<std::string>& config_output)
 {
 	for (const auto& color : fg_colors) {
 		std::string configline = strprintf::fmt("color %s %s %s",
-			color.first,
-			color.second,
-			bg_colors[color.first]);
+				color.first,
+				color.second,
+				bg_colors[color.first]);
 		for (const auto& attrib : attributes[color.first]) {
 			configline.append(" ");
 			configline.append(attrib);
@@ -118,14 +119,16 @@ void ColorManager::set_pb_colors(podboat::PbView* v)
 			colorattr.append(fgcit->second);
 		}
 		if (bgcit->second != "default") {
-			if (colorattr.length() > 0)
+			if (colorattr.length() > 0) {
 				colorattr.append(",");
+			}
 			colorattr.append("bg=");
 			colorattr.append(bgcit->second);
 		}
 		for (const auto& attr : attit->second) {
-			if (colorattr.length() > 0)
+			if (colorattr.length() > 0) {
 				colorattr.append(",");
+			}
 			colorattr.append("attr=");
 			colorattr.append(attr);
 		}
@@ -145,8 +148,9 @@ void ColorManager::set_pb_colors(podboat::PbView* v)
 				styleend_str.append("bg=");
 				styleend_str.append(bgcit->second);
 			}
-			if (styleend_str.length() > 0)
+			if (styleend_str.length() > 0) {
 				styleend_str.append(",");
+			}
 			styleend_str.append("attr=bold");
 
 			v->help_form.set("styleend", styleend_str.c_str());
