@@ -1,5 +1,6 @@
 #include "configparser.h"
 
+#include <algorithm>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
@@ -7,7 +8,8 @@
 #include <sys/types.h>
 
 #include "config.h"
-#include "exceptions.h"
+#include "configexception.h"
+#include "confighandlerexception.h"
 #include "logger.h"
 #include "strprintf.h"
 #include "tagsouppullparser.h"
@@ -35,7 +37,9 @@ void ConfigParser::handle_action(const std::string& action,
 				ActionHandlerStatus::TOO_FEW_PARAMS);
 		}
 
-		if (!this->parse(utils::resolve_tilde(params[0])))
+		std::string tilde_expanded = utils::resolve_tilde(params[0]);
+		std::string current_fpath = included_files.back();
+		if (!this->parse(utils::resolve_relative(current_fpath, tilde_expanded)))
 			throw ConfigHandlerException(
 				ActionHandlerStatus::FILENOTFOUND);
 	} else
@@ -43,7 +47,7 @@ void ConfigParser::handle_action(const std::string& action,
 			ActionHandlerStatus::INVALID_COMMAND);
 }
 
-bool ConfigParser::parse(const std::string& filename, bool double_include)
+bool ConfigParser::parse(const std::string& tmp_filename)
 {
 	/*
 	 * this function parses a config file.
@@ -59,15 +63,21 @@ bool ConfigParser::parse(const std::string& filename, bool double_include)
 	 *   - hand over the tokenize results to the ConfigActionHandler
 	 *   - if an error happens, react accordingly.
 	 */
-	if (!double_include &&
-		included_files.find(filename) != included_files.end()) {
+
+	// It would be nice if this function was only give absolute paths, but the
+	// tests are easier as relative paths
+	const std::string filename = (tmp_filename.front() == '/') ? tmp_filename :
+		utils::getcwd() + '/' + tmp_filename;
+
+	if (std::find(included_files.begin(), included_files.end(),
+			filename) != included_files.end()) {
 		LOG(Level::WARN,
 			"ConfigParser::parse: file %s has already been "
 			"included",
 			filename);
 		return true;
 	}
-	included_files.insert(included_files.begin(), filename);
+	included_files.push_back(filename);
 
 	unsigned int linecounter = 0;
 	std::ifstream f(filename.c_str());
@@ -78,11 +88,16 @@ bool ConfigParser::parse(const std::string& filename, bool double_include)
 			filename);
 		return false;
 	}
+
 	while (f.is_open() && !f.eof()) {
 		getline(f, line);
 		++linecounter;
 		LOG(Level::DEBUG, "ConfigParser::parse: tokenizing %s", line);
-		std::vector<std::string> tokens = utils::tokenize_quoted(line);
+
+		auto stripped = utils::strip_comments(line);
+		auto evaluated = evaluate_backticks(std::move(stripped));
+		std::vector<std::string> tokens = utils::tokenize_quoted(std::move(evaluated));
+
 		if (!tokens.empty()) {
 			std::string cmd = tokens[0];
 			ConfigActionHandler* handler = action_handlers[cmd];
@@ -90,24 +105,24 @@ bool ConfigParser::parse(const std::string& filename, bool double_include)
 				tokens.erase(
 					tokens.begin()); // delete first element
 				try {
-					evaluate_backticks(tokens);
 					handler->handle_action(cmd, tokens);
 				} catch (const ConfigHandlerException& e) {
 					throw ConfigException(strprintf::fmt(
-						_("Error while processing "
-						  "command `%s' (%s line %u): "
-						  "%s"),
-						line,
-						filename,
-						linecounter,
-						e.what()));
+							_("Error while processing "
+								"command `%s' (%s line %u): "
+								"%s"),
+							line,
+							filename,
+							linecounter,
+							e.what()));
 				}
 			} else {
 				throw ConfigException(strprintf::fmt(
-					_("unknown command `%s'"), cmd));
+						_("unknown command `%s'"), cmd));
 			}
 		}
 	}
+	included_files.pop_back();
 	return true;
 }
 
@@ -135,8 +150,9 @@ void ConfigParser::evaluate_backticks(std::vector<std::string>& tokens)
 std::string::size_type find_non_escaped_backtick(std::string& input,
 	const std::string::size_type startpos)
 {
-	if (startpos == std::string::npos)
+	if (startpos == std::string::npos) {
 		return startpos;
+	}
 
 	std::string::size_type result = startpos;
 	result = input.find_first_of("`", result);
@@ -168,7 +184,7 @@ std::string ConfigParser::evaluate_backticks(std::string token)
 		token.insert(pos1, result);
 
 		pos1 = find_non_escaped_backtick(
-			token, pos1 + result.length() + 1);
+				token, pos1 + result.length() + 1);
 		pos2 = find_non_escaped_backtick(token, pos1 + 1);
 	}
 
