@@ -8,18 +8,18 @@
 #include <string>
 
 #include "config.h"
-#include "exceptions.h"
+#include "dbexception.h"
 #include "feedcontainer.h"
-#include "formatstring.h"
+#include "fmtstrformatter.h"
 #include "listformatter.h"
 #include "logger.h"
 #include "reloader.h"
+#include "rssfeed.h"
+#include "scopemeasure.h"
 #include "strprintf.h"
 #include "utils.h"
 #include "view.h"
 
-#define FILTER_UNREAD_FEEDS "unread_count != \"0\""
-
 namespace newsboat {
 
 FeedListFormAction::FeedListFormAction(View* vv,
@@ -41,11 +41,11 @@ FeedListFormAction::FeedListFormAction(View* vv,
 	, total_feeds(0)
 	, filters(f)
 {
-	assert(true == m.parse(FILTER_UNREAD_FEEDS));
 	valid_cmds.push_back("tag");
 	valid_cmds.push_back("goto");
 	std::sort(valid_cmds.begin(), valid_cmds.end());
 	old_sort_order = cfg->get_configvalue("feed-sort-order");
+	search_dummy_feed->set_search_feed(true);
 }
 
 void FeedListFormAction::init()
@@ -66,8 +66,6 @@ void FeedListFormAction::init()
 	 * DownloadThreads.
 	 */
 	v->get_ctrl()->get_reloader()->spawn_reloadthread();
-
-	apply_filter = !(cfg->get_configvalue_as_bool("show-read-feeds"));
 }
 
 FeedListFormAction::~FeedListFormAction() {}
@@ -119,12 +117,12 @@ REDO:
 			if (feeds_shown > 0 && feedpos.length() > 0) {
 				v->push_itemlist(pos);
 			} else {
-				v->show_error(_("No feed selected!")); // should
-								       // not
-								       // happen
+				// should not happen
+				v->show_error(_("No feed selected!"));
 			}
 		}
-	} break;
+	}
+	break;
 	case OP_RELOAD: {
 		LOG(Level::INFO,
 			"FeedListFormAction: reloading feed at position `%s'",
@@ -135,7 +133,8 @@ REDO:
 			v->show_error(
 				_("No feed selected!")); // should not happen
 		}
-	} break;
+	}
+	break;
 	case OP_INT_RESIZE:
 		do_redraw = true;
 		break;
@@ -148,15 +147,24 @@ REDO:
 		/// (f)irsttag/..." messages
 		std::string input_options = _("ftauln");
 		char c = v->confirm(
-			_("Sort by "
-			  "(f)irsttag/(t)itle/(a)rticlecount/"
-			  "(u)nreadarticlecount/(l)astupdated/(n)one?"),
-			input_options);
-		if (!c)
+				_("Sort by "
+					"(f)irsttag/(t)itle/(a)rticlecount/"
+					"(u)nreadarticlecount/(l)astupdated/(n)one?"),
+				input_options);
+		if (!c) {
 			break;
-		unsigned int n_options = ((std::string) "ftaun").length();
-		if (input_options.length() < n_options)
+		}
+
+		// Check that the number of translated answers is the same as the
+		// number of answers we expect to handle. If it doesn't, just give up.
+		// That'll prevent this function from sorting anything, so users will
+		// complain, and we'll ask them to update the translation. A bit lame,
+		// but it's better than mishandling the answer.
+		const auto n_options = ((std::string) "ftaun").length();
+		if (input_options.length() < n_options) {
 			break;
+		}
+
 		if (c == input_options.at(0)) {
 			cfg->set_configvalue(
 				"feed-sort-order", "firsttag-desc");
@@ -174,19 +182,29 @@ REDO:
 		} else if (c == input_options.at(5)) {
 			cfg->set_configvalue("feed-sort-order", "none-desc");
 		}
-	} break;
+	}
+	break;
 	case OP_REVSORT: {
 		std::string input_options = _("ftauln");
 		char c = v->confirm(
-			_("Reverse Sort by "
-			  "(f)irsttag/(t)itle/(a)rticlecount/"
-			  "(u)nreadarticlecount/(l)astupdated/(n)one?"),
-			input_options);
-		if (!c)
+				_("Reverse Sort by "
+					"(f)irsttag/(t)itle/(a)rticlecount/"
+					"(u)nreadarticlecount/(l)astupdated/(n)one?"),
+				input_options);
+		if (!c) {
 			break;
-		unsigned int n_options = ((std::string) "ftaun").length();
-		if (input_options.length() < n_options)
+		}
+
+		// Check that the number of translated answers is the same as the
+		// number of answers we expect to handle. If it doesn't, just give up.
+		// That'll prevent this function from sorting anything, so users will
+		// complain, and we'll ask them to update the translation. A bit lame,
+		// but it's better than mishandling the answer.
+		const auto n_options = ((std::string) "ftaun").length();
+		if (input_options.length() < n_options) {
 			break;
+		}
+
 		if (c == input_options.at(0)) {
 			cfg->set_configvalue("feed-sort-order", "firsttag-asc");
 		} else if (c == input_options.at(1)) {
@@ -203,7 +221,8 @@ REDO:
 		} else if (c == input_options.at(5)) {
 			cfg->set_configvalue("feed-sort-order", "none-asc");
 		}
-	} break;
+	}
+	break;
 	case OP_OPENINBROWSER:
 		if (feeds_shown > 0 && feedpos.length() > 0) {
 			std::shared_ptr<RssFeed> feed =
@@ -234,7 +253,7 @@ REDO:
 				} else {
 					v->show_error(
 						_("Cannot open query feeds in "
-						  "the browser!"));
+							"the browser!"));
 				}
 			}
 		} else {
@@ -300,30 +319,29 @@ REDO:
 				v->get_ctrl()->mark_all_read(pos);
 				do_redraw = true;
 				v->set_status("");
-				if (feeds_shown > (pos + 1) && !apply_filter) {
+				bool show_read = cfg->get_configvalue_as_bool("show-read-feeds");
+				if (feeds_shown > (pos + 1) && show_read) {
 					f->set("feedpos",
 						std::to_string(pos + 1));
 				}
 			} catch (const DbException& e) {
 				v->show_error(strprintf::fmt(
-					_("Error: couldn't mark feed read: %s"),
-					e.what()));
+						_("Error: couldn't mark feed read: %s"),
+						e.what()));
 			}
 		} else {
 			v->show_error(
 				_("No feed selected!")); // should not happen
 		}
-	} break;
+	}
+	break;
 	case OP_TOGGLESHOWREAD:
-		m.parse(FILTER_UNREAD_FEEDS);
 		LOG(Level::INFO,
 			"FeedListFormAction: toggling show-read-feeds");
 		if (cfg->get_configvalue_as_bool("show-read-feeds")) {
 			cfg->set_configvalue("show-read-feeds", "no");
-			apply_filter = true;
 		} else {
 			cfg->set_configvalue("show-read-feeds", "yes");
-			apply_filter = false;
 		}
 		save_filterpos();
 		do_redraw = true;
@@ -335,7 +353,8 @@ REDO:
 		if (!jump_to_next_unread_feed(local_tmp)) {
 			v->show_error(_("No feeds with unread items."));
 		}
-	} break;
+	}
+	break;
 	case OP_PREVUNREAD: {
 		unsigned int local_tmp;
 		LOG(Level::INFO,
@@ -343,14 +362,16 @@ REDO:
 		if (!jump_to_previous_unread_feed(local_tmp)) {
 			v->show_error(_("No feeds with unread items."));
 		}
-	} break;
+	}
+	break;
 	case OP_NEXT: {
 		unsigned int local_tmp;
 		LOG(Level::INFO, "FeedListFormAction: jumping to next feed");
 		if (!jump_to_next_feed(local_tmp)) {
 			v->show_error(_("Already on last feed."));
 		}
-	} break;
+	}
+	break;
 	case OP_PREV: {
 		unsigned int local_tmp;
 		LOG(Level::INFO,
@@ -358,7 +379,8 @@ REDO:
 		if (!jump_to_previous_feed(local_tmp)) {
 			v->show_error(_("Already on first feed."));
 		}
-	} break;
+	}
+	break;
 	case OP_RANDOMUNREAD: {
 		unsigned int local_tmp;
 		LOG(Level::INFO,
@@ -366,7 +388,8 @@ REDO:
 		if (!jump_to_random_unread_feed(local_tmp)) {
 			v->show_error(_("No feeds with unread items."));
 		}
-	} break;
+	}
+	break;
 	case OP_MARKALLFEEDSREAD:
 		LOG(Level::INFO, "FeedListFormAction: marking all feeds read");
 		v->set_status(_("Marking all feeds read..."));
@@ -401,7 +424,8 @@ REDO:
 			do_redraw = true;
 			zero_feedpos = true;
 		}
-	} break;
+	}
+	break;
 	case OP_SELECTFILTER:
 		if (filters->size() > 0) {
 			std::string newfilter;
@@ -409,19 +433,18 @@ REDO:
 				newfilter = (*args)[0];
 			} else {
 				newfilter = v->select_filter(
-					filters->get_filters());
+						filters->get_filters());
 			}
 			if (newfilter != "") {
 				filterhistory.add_line(newfilter);
 				if (newfilter.length() > 0) {
 					if (!m.parse(newfilter)) {
 						v->show_error(strprintf::fmt(
-							_("Error: couldn't "
-							  "parse filter "
-							  "command `%s': %s"),
-							newfilter,
-							m.get_parse_error()));
-						m.parse(FILTER_UNREAD_FEEDS);
+								_("Error: couldn't "
+									"parse filter "
+									"command `%s': %s"),
+								newfilter,
+								m.get_parse_error()));
 					} else {
 						save_filterpos();
 						apply_filter = true;
@@ -450,9 +473,7 @@ REDO:
 		}
 		break;
 	case OP_CLEARFILTER:
-		apply_filter =
-			!(cfg->get_configvalue_as_bool("show-read-feeds"));
-		m.parse(FILTER_UNREAD_FEEDS);
+		apply_filter = false;
 		do_redraw = true;
 		save_filterpos();
 		break;
@@ -497,8 +518,9 @@ REDO:
 		break;
 	}
 	if (quit) {
-		while (v->formaction_stack_size() > 0)
+		while (v->formaction_stack_size() > 0) {
 			v->pop_current_formaction();
+		}
 	}
 }
 
@@ -509,11 +531,13 @@ void FeedListFormAction::update_visible_feeds(
 
 	visible_feeds.clear();
 
-	unsigned int i = 0;
+	bool show_read = cfg->get_configvalue_as_bool("show-read-feeds");
 
+	unsigned int i = 0;
 	for (const auto& feed : feeds) {
 		feed->set_index(i + 1);
 		if ((tag == "" || feed->matches_tag(tag)) &&
+			(show_read || feed->unread_item_count() > 0) &&
 			(!apply_filter || m.matches(feed.get())) &&
 			!feed->hidden()) {
 			visible_feeds.push_back(FeedPtrPosPair(feed, i));
@@ -541,13 +565,14 @@ void FeedListFormAction::set_feedlist(
 	update_visible_feeds(feeds);
 
 	for (const auto& feed : visible_feeds) {
-		if (feed.first->unread_item_count() > 0)
+		if (feed.first->unread_item_count() > 0) {
 			++unread_feeds;
+		}
 
 		listfmt.add_line(format_line(feedlist_format,
-					 feed.first,
-					 feed.second,
-					 width),
+				feed.first,
+				feed.second,
+				width),
 			feed.second);
 		i++;
 	}
@@ -564,7 +589,7 @@ void FeedListFormAction::set_feedlist(
 	FmtStrFormatter fmt;
 	fmt.register_fmt('T', tag);
 	fmt.register_fmt('N', PROGRAM_NAME);
-	fmt.register_fmt('V', PROGRAM_VERSION);
+	fmt.register_fmt('V', utils::program_version());
 	fmt.register_fmt('u', std::to_string(unread_feeds));
 	fmt.register_fmt('t', std::to_string(i));
 
@@ -587,7 +612,8 @@ KeyMapHintEntry* FeedListFormAction::get_keymap_hint()
 		{OP_MARKALLFEEDSREAD, _("Mark All Read")},
 		{OP_SEARCH, _("Search")},
 		{OP_HELP, _("Help")},
-		{OP_NIL, nullptr}};
+		{OP_NIL, nullptr}
+	};
 	return hints;
 }
 
@@ -644,14 +670,14 @@ void FeedListFormAction::goto_feed(const std::string& str)
 		str);
 	for (unsigned int i = curpos + 1; i < visible_feeds.size(); ++i) {
 		if (strcasestr(visible_feeds[i].first->title().c_str(),
-			    str.c_str()) != nullptr) {
+				str.c_str()) != nullptr) {
 			f->set("feedpos", std::to_string(i));
 			return;
 		}
 	}
 	for (unsigned int i = 0; i <= curpos; ++i) {
 		if (strcasestr(visible_feeds[i].first->title().c_str(),
-			    str.c_str()) != nullptr) {
+				str.c_str()) != nullptr) {
 			f->set("feedpos", std::to_string(i));
 			return;
 		}
@@ -766,8 +792,9 @@ std::shared_ptr<RssFeed> FeedListFormAction::get_feed()
 int FeedListFormAction::get_pos(unsigned int realidx)
 {
 	for (unsigned int i = 0; i < visible_feeds.size(); ++i) {
-		if (visible_feeds[i].second == realidx)
+		if (visible_feeds[i].second == realidx) {
 			return i;
+		}
 	}
 	return -1;
 }
@@ -878,10 +905,10 @@ void FeedListFormAction::set_regexmanager(RegexManager* r)
 		i++;
 	}
 	std::string textview = strprintf::fmt(
-		"{!list[feeds] .expand:vh style_normal[listnormal]: "
-		"style_focus[listfocus]:fg=yellow,bg=blue,attr=bold "
-		"pos_name[feedposname]: pos[feedpos]:0 %s richtext:1}",
-		attrstr);
+			"{!list[feeds] .expand:vh style_normal[listnormal]: "
+			"style_focus[listfocus]:fg=yellow,bg=blue,attr=bold "
+			"pos_name[feedposname]: pos[feedpos]:0 %s richtext:1}",
+			attrstr);
 	f->modify("feeds", "replace", textview);
 }
 
@@ -893,7 +920,6 @@ void FeedListFormAction::op_end_setfilter()
 		if (!m.parse(filtertext)) {
 			v->show_error(
 				_("Error: couldn't parse filter command!"));
-			m.parse(FILTER_UNREAD_FEEDS);
 		} else {
 			save_filterpos();
 			apply_filter = true;
@@ -915,14 +941,14 @@ void FeedListFormAction::op_start_search()
 		std::vector<std::shared_ptr<RssItem>> items;
 		try {
 			std::string utf8searchphrase = utils::convert_text(
-				searchphrase, "utf-8", nl_langinfo(CODESET));
+					searchphrase, "utf-8", nl_langinfo(CODESET));
 			items = v->get_ctrl()->search_for_items(
-				utf8searchphrase, nullptr);
+					utf8searchphrase, nullptr);
 		} catch (const DbException& e) {
 			v->show_error(strprintf::fmt(
-				_("Error while searching for `%s': %s"),
-				searchphrase,
-				e.what()));
+					_("Error while searching for `%s': %s"),
+					searchphrase,
+					e.what()));
 			return;
 		}
 		if (!items.empty()) {
@@ -975,10 +1001,12 @@ std::string FeedListFormAction::get_title(std::shared_ptr<RssFeed> feed)
 {
 	std::string title = feed->title();
 	utils::remove_soft_hyphens(title);
-	if (title.length() == 0)
+	if (title.length() == 0) {
 		title = utils::censor_url(feed->rssurl());
-	if (title.length() == 0)
+	}
+	if (title.length() == 0) {
 		title = "<no title>";
+	}
 	return title;
 }
 
@@ -1016,8 +1044,8 @@ std::string FeedListFormAction::format_line(const std::string& feedlist_format,
 std::string FeedListFormAction::title()
 {
 	return strprintf::fmt(_("Feed List - %u unread, %u total"),
-		unread_feeds,
-		total_feeds);
+			unread_feeds,
+			total_feeds);
 }
 
 } // namespace newsboat
