@@ -1,6 +1,7 @@
 #include "filebrowserformaction.h"
 
 #include <algorithm>
+#include <cinttypes>
 #include <cstring>
 #include <curses.h>
 #include <dirent.h>
@@ -14,7 +15,7 @@
 #include <unistd.h>
 
 #include "config.h"
-#include "formatstring.h"
+#include "fmtstrformatter.h"
 #include "logger.h"
 #include "strprintf.h"
 #include "utils.h"
@@ -68,7 +69,7 @@ void FileBrowserFormAction::process_operation(Operation op,
 
 					FmtStrFormatter fmt;
 					fmt.register_fmt('N', PROGRAM_NAME);
-					fmt.register_fmt('V', PROGRAM_VERSION);
+					fmt.register_fmt('V', utils::program_version());
 					fmt.register_fmt('f', filename);
 					f->set("head",
 						fmt.do_format(
@@ -103,14 +104,16 @@ void FileBrowserFormAction::process_operation(Operation op,
 					}
 					f->set("filenametext", fn);
 					do_redraw = true;
-				} break;
+				}
+				break;
 				case '-': {
 					std::string fn = utils::getcwd();
 					fn.append(NEWSBEUTER_PATH_SEP);
 					fn.append(filename);
 					f->set("filenametext", fn);
 					f->set_focus("filename");
-				} break;
+				}
+				break;
 				default:
 					// TODO: show error message
 					break;
@@ -127,13 +130,13 @@ void FileBrowserFormAction::process_operation(Operation op,
 				if (::stat(fn.c_str(), &sbuf) != -1) {
 					f->set_focus("files");
 					if (v->confirm(
-						    strprintf::fmt(
-							    _("Do you really "
-							      "want to "
-							      "overwrite `%s' "
-							      "(y:Yes n:No)? "),
-							    fn),
-						    _("yn")) == *_("n")) {
+							strprintf::fmt(
+								_("Do you really "
+									"want to "
+									"overwrite `%s' "
+									"(y:Yes n:No)? "),
+								fn),
+							_("yn")) == *_("n")) {
 						do_pop = false;
 					}
 					f->set_focus("filenametext");
@@ -144,7 +147,8 @@ void FileBrowserFormAction::process_operation(Operation op,
 				}
 			}
 		}
-	} break;
+	}
+	break;
 	case OP_QUIT:
 		LOG(Level::DEBUG, "view::filebrowser: quitting");
 		curs_set(0);
@@ -174,8 +178,9 @@ std::vector<std::string> get_sorted_filelist()
 		struct dirent* de = ::readdir(dirp);
 		while (de) {
 			if (strcmp(de->d_name, ".") != 0 &&
-				strcmp(de->d_name, "..") != 0)
+				strcmp(de->d_name, "..") != 0) {
 				ret.push_back(de->d_name);
+			}
 			de = ::readdir(dirp);
 		}
 		::closedir(dirp);
@@ -183,8 +188,9 @@ std::vector<std::string> get_sorted_filelist()
 
 	std::sort(ret.begin(), ret.end());
 
-	if (cwdtmp != "/")
+	if (cwdtmp != "/") {
 		ret.insert(ret.begin(), "..");
+	}
 
 	return ret;
 }
@@ -249,7 +255,7 @@ void FileBrowserFormAction::init()
 	f->set("head",
 		strprintf::fmt(_("%s %s - Save File - %s"),
 			PROGRAM_NAME,
-			PROGRAM_VERSION,
+			utils::program_version(),
 			cwdtmp));
 }
 
@@ -257,7 +263,8 @@ KeyMapHintEntry* FileBrowserFormAction::get_keymap_hint()
 {
 	static KeyMapHintEntry hints[] = {{OP_QUIT, _("Cancel")},
 		{OP_OPEN, _("Save")},
-		{OP_NIL, nullptr}};
+		{OP_NIL, nullptr}
+	};
 	return hints;
 }
 
@@ -274,18 +281,23 @@ std::string FileBrowserFormAction::add_file(std::string filename)
 		std::string formattedfilename =
 			get_formatted_filename(filename, ftype, sb.st_mode);
 
-		std::string sizestr = strprintf::fmt("%12u", sb.st_size);
+		std::string sizestr = strprintf::fmt(
+				"%12" PRIi64,
+				// `st_size` is `off_t`, which is a signed integer type of
+				// unspecified size. We'll have to bet it's no larger than 64
+				// bits.
+				static_cast<int64_t>(sb.st_size));
 		std::string line = strprintf::fmt("%c%s %s %s %s %s",
-			ftype,
-			rwxbits,
-			owner,
-			group,
-			sizestr,
-			formattedfilename);
+				ftype,
+				rwxbits,
+				owner,
+				group,
+				sizestr,
+				formattedfilename);
 		retval = strprintf::fmt("{listitem[%c%s] text:%s}",
-			ftype,
-			Stfl::quote(filename),
-			Stfl::quote(line));
+				ftype,
+				Stfl::quote(filename),
+				Stfl::quote(line));
 	}
 	return retval;
 }
@@ -310,8 +322,9 @@ std::string FileBrowserFormAction::get_formatted_filename(std::string filename,
 		suffix = '|';
 		break;
 	default:
-		if (mode & S_IXUSR)
+		if (mode & S_IXUSR) {
 			suffix = '*';
+		}
 	}
 
 	return strprintf::fmt("%s%c", filename, suffix);
@@ -321,7 +334,8 @@ std::string FileBrowserFormAction::get_rwx(unsigned short val)
 {
 	std::string str;
 	const char* bitstrs[] = {
-		"---", "--x", "-w-", "-wx", "r--", "r-x", "rw-", "rwx"};
+		"---", "--x", "-w-", "-wx", "r--", "r-x", "rw-", "rwx"
+	};
 	for (int i = 0; i < 3; ++i) {
 		unsigned char bits = val % 8;
 		val /= 8;
@@ -342,10 +356,12 @@ char FileBrowserFormAction::get_filetype(mode_t mode)
 		{S_IFIFO, 'p'},
 		{S_IFLNK, 'l'},
 		{S_IFSOCK, 's'},
-		{0, 0}};
+		{0, 0}
+	};
 	for (unsigned int i = 0; flags[i].flag != 0; i++) {
-		if ((mode & S_IFMT) == flags[i].flag)
+		if ((mode & S_IFMT) == flags[i].flag) {
 			return flags[i].ftype;
+		}
 	}
 	return '?';
 }
