@@ -24,28 +24,32 @@ void FileUrlReader::reload()
 
 	std::fstream f;
 	f.open(filename.c_str(), std::fstream::in);
-	if (f.is_open()) {
-		std::string line;
-		while (!f.eof()) {
-			std::getline(f, line);
-			if (line.length() > 0 && line[0] != '#') {
-				std::vector<std::string> tokens =
-					utils::tokenize_quoted(line);
-				if (!tokens.empty()) {
-					std::string url = tokens[0];
-					urls.push_back(url);
-					tokens.erase(tokens.begin());
-					if (!tokens.empty()) {
-						tags[url] = tokens;
-						for (const auto& token :
-							tokens) {
-							alltags.insert(token);
-						}
-					}
-				}
-			}
-		};
+	if (!f.is_open()) {
+		return;
 	}
+
+	for (std::string line; std::getline(f, line); /* nothing */) {
+		// skip empty lines and comments
+		if (line.empty() || line[0] == '#') {
+			continue;
+		}
+
+		std::vector<std::string> tokens = utils::tokenize_quoted(line);
+		if (tokens.empty()) {
+			continue;
+		}
+
+		std::string url = tokens[0];
+		urls.push_back(url);
+
+		tokens.erase(tokens.begin());
+		if (!tokens.empty()) {
+			tags[url] = tokens;
+			for (const auto& token : tokens) {
+				alltags.insert(token);
+			}
+		}
+	};
 }
 
 void FileUrlReader::load_config(const std::string& file)
