@@ -1,7 +1,7 @@
 #include "filtercontainer.h"
 
 #include "config.h"
-#include "exceptions.h"
+#include "confighandlerexception.h"
 #include "matcher.h"
 #include "strprintf.h"
 #include "utils.h"
@@ -26,9 +26,9 @@ void FilterContainer::handle_action(const std::string& action,
 		Matcher m;
 		if (!m.parse(params[1]))
 			throw ConfigHandlerException(strprintf::fmt(
-				_("couldn't parse filter expression `%s': %s"),
-				params[1],
-				m.get_parse_error()));
+					_("couldn't parse filter expression `%s': %s"),
+					params[1],
+					m.get_parse_error()));
 		filters.push_back(FilterNameExprPair(params[0], params[1]));
 	} else
 		throw ConfigHandlerException(
@@ -39,8 +39,8 @@ void FilterContainer::dump_config(std::vector<std::string>& config_output)
 {
 	for (const auto& filter : filters) {
 		config_output.push_back(strprintf::fmt("define-filter %s %s",
-			utils::quote(filter.first),
-			utils::quote(filter.second)));
+				utils::quote(filter.first),
+				utils::quote(filter.second)));
 	}
 }
 
