@@ -1,6 +1,7 @@
 #include "htmlrenderer.h"
 
 #include <algorithm>
+#include <cinttypes>
 #include <cstdio>
 #include <cstring>
 #include <iostream>
@@ -77,8 +78,9 @@ unsigned int HtmlRenderer::add_link(std::vector<LinkPair>& links,
 		}
 		i++;
 	}
-	if (!found)
+	if (!found) {
 		links.push_back(LinkPair(link, type));
+	}
 
 	return i;
 }
@@ -140,25 +142,30 @@ void HtmlRenderer::render(std::istream& input,
 				}
 				if (link.length() > 0) {
 					link_num = add_link(links,
-						utils::censor_url(
-							utils::absolute_url(
-								url, link)),
-						LinkType::HREF);
-					if (!raw_)
+							utils::censor_url(
+								utils::absolute_url(
+									url, link)),
+							LinkType::HREF);
+					if (!raw_) {
 						curline.append("<u>");
+					}
 				}
-			} break;
+			}
+			break;
 			case HtmlTag::STRONG:
-				if (!raw_)
+				if (!raw_) {
 					curline.append("<b>");
+				}
 				break;
 			case HtmlTag::UNDERLINE:
-				if (!raw_)
+				if (!raw_) {
 					curline.append("<u>");
+				}
 				break;
 			case HtmlTag::QUOTATION:
-				if (!raw_)
+				if (!raw_) {
 					curline.append("\"");
+				}
 				break;
 
 			case HtmlTag::EMBED: {
@@ -177,7 +184,7 @@ void HtmlRenderer::render(std::istream& input,
 					std::string link;
 					try {
 						link = xpp.get_attribute_value(
-							"src");
+								"src");
 					} catch (const std::invalid_argument&) {
 						LOG(Level::WARN,
 							"HtmlRenderer::render: "
@@ -188,18 +195,19 @@ void HtmlRenderer::render(std::istream& input,
 					}
 					if (link.length() > 0) {
 						link_num = add_link(links,
-							utils::censor_url(
-								utils::absolute_url(
-									url,
-									link)),
-							LinkType::EMBED);
+								utils::censor_url(
+									utils::absolute_url(
+										url,
+										link)),
+								LinkType::EMBED);
 						curline.append(strprintf::fmt(
-							"[%s %u]",
-							_("embedded flash:"),
-							link_num));
+								"[%s %u]",
+								_("embedded flash:"),
+								link_num));
 					}
 				}
-			} break;
+			}
+			break;
 
 			case HtmlTag::BR:
 				add_line(curline, tables, lines);
@@ -232,38 +240,39 @@ void HtmlRenderer::render(std::istream& input,
 				}
 				try {
 					imgtitle = xpp.get_attribute_value(
-						"title");
+							"title");
 				} catch (const std::invalid_argument&) {
 					imgtitle = "";
 				}
 				if (imgurl.length() > 0) {
 					if (imgurl.substr(0, 5) == "data:") {
 						link_num = add_link(links,
-							"inline image",
-							LinkType::IMG);
+								"inline image",
+								LinkType::IMG);
 					} else {
 						link_num = add_link(links,
-							utils::censor_url(
-								utils::absolute_url(
-									url,
-									imgurl)),
-							LinkType::IMG);
+								utils::censor_url(
+									utils::absolute_url(
+										url,
+										imgurl)),
+								LinkType::IMG);
 					}
 					if (imgtitle != "") {
 						curline.append(strprintf::fmt(
-							"[%s %u: %s]",
-							_("image"),
-							link_num,
-							imgtitle));
+								"[%s %u: %s]",
+								_("image"),
+								link_num,
+								imgtitle));
 					} else {
 						curline.append(strprintf::fmt(
-							"[%s %u]",
-							_("image"),
-							link_num));
+								"[%s %u]",
+								_("image"),
+								link_num));
 					}
 					image_count++;
 				}
-			} break;
+			}
+			break;
 
 			case HtmlTag::BLOCKQUOTE:
 				++indent_level;
@@ -284,15 +293,17 @@ void HtmlRenderer::render(std::istream& input,
 				if (lines.size() > 0) {
 					std::string::size_type last_line_len =
 						lines[lines.size() - 1]
-							.second.length();
+						.second.length();
 					if (last_line_len >
 						static_cast<unsigned int>(
-							indent_level * 2))
+							indent_level * 2)) {
 						add_line("", tables, lines);
+					}
 				}
 				prepare_new_line(curline,
 					tables.size() ? 0 : indent_level);
-			} break;
+			}
+			break;
 
 			case HtmlTag::OL:
 				is_ol = true;
@@ -343,13 +354,14 @@ void HtmlRenderer::render(std::istream& input,
 			case HtmlTag::LI:
 				if (inside_li) {
 					indent_level -= 2;
-					if (indent_level < 0)
+					if (indent_level < 0) {
 						indent_level = 0;
+					}
 					add_nonempty_line(
 						curline, tables, lines);
 					prepare_new_line(curline,
 						tables.size() ? 0
-							      : indent_level);
+						: indent_level);
 				}
 				inside_li = true;
 				add_nonempty_line(curline, tables, lines);
@@ -358,12 +370,12 @@ void HtmlRenderer::render(std::istream& input,
 				indent_level += 2;
 				if (is_ol && ol_counts.size() != 0) {
 					curline.append(strprintf::fmt("%s. ",
-						format_ol_count(
-							ol_counts[ol_counts
-									  .size() -
-								1],
-							ol_types[ol_types.size() -
-								1])));
+							format_ol_count(
+								ol_counts[ol_counts
+									.size() -
+									1],
+								ol_types[ol_types.size() -
+											1])));
 					++ol_counts[ol_counts.size() - 1];
 				} else {
 					curline.append("  * ");
@@ -423,7 +435,7 @@ void HtmlRenderer::render(std::istream& input,
 				bool has_border = false;
 				try {
 					std::string b = xpp.get_attribute_value(
-						"border");
+							"border");
 					has_border = (utils::to_u(b, 0) > 0);
 				} catch (const std::invalid_argument&) {
 					// is ok, no border then
@@ -433,22 +445,24 @@ void HtmlRenderer::render(std::istream& input,
 			}
 
 			case HtmlTag::TR:
-				if (!tables.empty())
+				if (!tables.empty()) {
 					tables.back().start_row();
+				}
 				break;
 
 			case HtmlTag::TH: {
 				size_t span = 1;
 				try {
 					span = utils::to_u(
-						xpp.get_attribute_value(
-							"colspan"),
-						1);
+							xpp.get_attribute_value(
+								"colspan"),
+							1);
 				} catch (const std::invalid_argument&) {
 					// is ok, span 1 then
 				}
-				if (!tables.empty())
+				if (!tables.empty()) {
 					tables.back().start_cell(span);
+				}
 				curline.append("<b>");
 				break;
 			}
@@ -457,14 +471,15 @@ void HtmlRenderer::render(std::istream& input,
 				size_t span = 1;
 				try {
 					span = utils::to_u(
-						xpp.get_attribute_value(
-							"colspan"),
-						1);
+							xpp.get_attribute_value(
+								"colspan"),
+							1);
 				} catch (const std::invalid_argument&) {
 					// is ok, span 1 then
 				}
-				if (!tables.empty())
+				if (!tables.empty()) {
 					tables.back().start_cell(span);
+				}
 				break;
 			}
 			}
@@ -481,8 +496,9 @@ void HtmlRenderer::render(std::istream& input,
 			switch (current_tag) {
 			case HtmlTag::BLOCKQUOTE:
 				--indent_level;
-				if (indent_level < 0)
+				if (indent_level < 0) {
 					indent_level = 0;
+				}
 				add_nonempty_line(curline, tables, lines);
 				add_line("", tables, lines);
 				prepare_new_line(curline,
@@ -496,13 +512,14 @@ void HtmlRenderer::render(std::istream& input,
 			case HtmlTag::UL:
 				if (inside_li) {
 					indent_level -= 2;
-					if (indent_level < 0)
+					if (indent_level < 0) {
 						indent_level = 0;
+					}
 					add_nonempty_line(
 						curline, tables, lines);
 					prepare_new_line(curline,
 						tables.size() ? 0
-							      : indent_level);
+						: indent_level);
 				}
 				add_nonempty_line(curline, tables, lines);
 				add_line("", tables, lines);
@@ -519,8 +536,9 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::DD:
 				indent_level -= 4;
-				if (indent_level < 0)
+				if (indent_level < 0) {
 					indent_level = 0;
+				}
 				add_nonempty_line(curline, tables, lines);
 				add_line("", tables, lines);
 				prepare_new_line(curline,
@@ -533,8 +551,9 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::LI:
 				indent_level -= 2;
-				if (indent_level < 0)
+				if (indent_level < 0) {
 					indent_level = 0;
+				}
 				inside_li = false;
 				add_nonempty_line(curline, tables, lines);
 				prepare_new_line(curline,
@@ -548,7 +567,7 @@ void HtmlRenderer::render(std::istream& input,
 						utils::strwidth_stfl(curline);
 					prepare_new_line(curline,
 						tables.size() ? 0
-							      : indent_level);
+						: indent_level);
 					add_line(std::string(llen, '-'),
 						tables,
 						lines);
@@ -585,27 +604,31 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::A:
 				if (link_num != -1) {
-					if (!raw_)
+					if (!raw_) {
 						curline.append("</>");
+					}
 					curline.append(strprintf::fmt(
-						"[%d]", link_num));
+							"[%d]", link_num));
 					link_num = -1;
 				}
 				break;
 
 			case HtmlTag::UNDERLINE:
-				if (!raw_)
+				if (!raw_) {
 					curline.append("</>");
+				}
 				break;
 
 			case HtmlTag::STRONG:
-				if (!raw_)
+				if (!raw_) {
 					curline.append("</>");
+				}
 				break;
 
 			case HtmlTag::QUOTATION:
-				if (!raw_)
+				if (!raw_) {
 					curline.append("\"");
+				}
 				break;
 
 			case HtmlTag::EMBED:
@@ -618,15 +641,17 @@ void HtmlRenderer::render(std::istream& input,
 
 			case HtmlTag::SCRIPT:
 				// don't render scripts, ignore current line
-				if (inside_script)
+				if (inside_script) {
 					inside_script--;
+				}
 				prepare_new_line(curline,
 					tables.size() ? 0 : indent_level);
 				break;
 
 			case HtmlTag::STYLE:
-				if (inside_style)
+				if (inside_style) {
 					inside_style--;
+				}
 				break;
 
 			case HtmlTag::TABLE:
@@ -636,32 +661,33 @@ void HtmlRenderer::render(std::istream& input,
 
 				if (!tables.empty()) {
 					std::vector<std::pair<LineType,
-						std::string>>
-						table_text;
+					    std::string>>
+					    table_text;
 					tables.back().complete_cell();
 					tables.back().complete_row();
 					render_table(tables.back(), table_text);
 					tables.pop_back();
 
-					if (!tables.empty()) { // still a table
-							       // on the
-							       // outside?
+					// still a table on the outside?
+					if (!tables.empty()) {
 						for (size_t idx = 0;
 							idx < table_text.size();
 							++idx)
+							// add rendered table to current cell
 							tables.back().add_text(
 								table_text[idx]
-									.second); // add rendered table to current cell
+								.second);
 					} else {
 						for (size_t idx = 0;
 							idx < table_text.size();
 							++idx) {
 							std::string s =
 								table_text[idx]
-									.second;
+								.second;
 							while (s.length() > 0 &&
-								s[0] == '\n')
+								s[0] == '\n') {
 								s.erase(0, 1);
+							}
 							add_line_nonwrappable(
 								s, lines);
 						}
@@ -676,8 +702,9 @@ void HtmlRenderer::render(std::istream& input,
 				prepare_new_line(
 					curline, 0); // no indent in tables
 
-				if (!tables.empty())
+				if (!tables.empty()) {
 					tables.back().complete_row();
+				}
 				break;
 
 			case HtmlTag::TH:
@@ -699,8 +726,9 @@ void HtmlRenderer::render(std::istream& input,
 				prepare_new_line(
 					curline, 0); // no indent in tables
 
-				if (!tables.empty())
+				if (!tables.empty()) {
 					tables.back().complete_cell();
+				}
 				break;
 			}
 			break;
@@ -716,8 +744,8 @@ void HtmlRenderer::render(std::istream& input,
 							curline, tables, lines);
 						prepare_new_line(curline,
 							tables.size()
-								? 0
-								: indent_level);
+							? 0
+							: indent_level);
 						curline.append(paragraph);
 					}
 				}
@@ -730,8 +758,8 @@ void HtmlRenderer::render(std::istream& input,
 							curline, lines);
 						prepare_new_line(curline,
 							tables.size()
-								? 0
-								: indent_level);
+							? 0
+							: indent_level);
 					} else {
 						curline.append(paragraph);
 					}
@@ -754,7 +782,8 @@ void HtmlRenderer::render(std::istream& input,
 				text = utils::replace_all(text, "\n", " ");
 				curline.append(text);
 			}
-		} break;
+		}
+		break;
 		default:
 			/* do nothing */
 			break;
@@ -771,8 +800,9 @@ void HtmlRenderer::render(std::istream& input,
 		tables.pop_back();
 		for (size_t idx = 0; idx < table_text.size(); ++idx) {
 			std::string s = table_text[idx].second;
-			while (s.length() > 0 && s[0] == '\n')
+			while (s.length() > 0 && s[0] == '\n') {
 				s.erase(0, 1);
+			}
 			add_line_nonwrappable(s, lines);
 		}
 	}
@@ -783,9 +813,9 @@ void HtmlRenderer::render(std::istream& input,
 		add_line(_("Links: "), tables, lines);
 		for (unsigned int i = 0; i < links.size(); ++i) {
 			auto link_text = strprintf::fmt("[%u]: %s (%s)",
-				i + 1,
-				links[i].first,
-				type2str(links[i].second));
+					i + 1,
+					links[i].first,
+					type2str(links[i].second));
 			add_line_softwrappable(link_text, lines);
 		}
 	}
@@ -818,8 +848,9 @@ void HtmlRenderer::add_nonempty_line(const std::string& curline,
 	std::vector<Table>& tables,
 	std::vector<std::pair<LineType, std::string>>& lines)
 {
-	if (line_is_nonempty(curline))
+	if (line_is_nonempty(curline)) {
 		add_line(curline, tables, lines);
+	}
 }
 
 void HtmlRenderer::add_hr(std::vector<std::pair<LineType, std::string>>& lines)
@@ -831,10 +862,11 @@ void HtmlRenderer::add_line(const std::string& curline,
 	std::vector<Table>& tables,
 	std::vector<std::pair<LineType, std::string>>& lines)
 {
-	if (tables.size())
+	if (tables.size()) {
 		tables.back().add_text(curline);
-	else
+	} else {
 		lines.push_back(std::make_pair(LineType::wrappable, curline));
+	}
 }
 
 void HtmlRenderer::add_line_softwrappable(const std::string& line,
@@ -858,8 +890,9 @@ void HtmlRenderer::prepare_new_line(std::string& line, int indent_level)
 bool HtmlRenderer::line_is_nonempty(const std::string& line)
 {
 	for (std::string::size_type i = 0; i < line.length(); ++i) {
-		if (!isblank(line[i]) && line[i] != '\n' && line[i] != '\r')
+		if (!isblank(line[i]) && line[i] != '\n' && line[i] != '\r') {
 			return true;
+		}
 	}
 	return false;
 }
@@ -867,15 +900,17 @@ bool HtmlRenderer::line_is_nonempty(const std::string& line)
 void HtmlRenderer::TableRow::start_cell(size_t span)
 {
 	inside = true;
-	if (span < 1)
+	if (span < 1) {
 		span = 1;
+	}
 	cells.push_back(TableCell(span));
 }
 
 void HtmlRenderer::TableRow::add_text(const std::string& str)
 {
-	if (!inside)
-		start_cell(1); // colspan 1
+	if (!inside) {
+		start_cell(1);        // colspan 1
+	}
 
 	cells.back().text.push_back(str);
 }
@@ -887,8 +922,9 @@ void HtmlRenderer::TableRow::complete_cell()
 
 void HtmlRenderer::Table::start_cell(size_t span)
 {
-	if (!inside)
+	if (!inside) {
 		start_row();
+	}
 	rows.back().start_cell(span);
 }
 
@@ -901,16 +937,18 @@ void HtmlRenderer::Table::complete_cell()
 
 void HtmlRenderer::Table::start_row()
 {
-	if (rows.size() && rows.back().inside)
+	if (rows.size() && rows.back().inside) {
 		rows.back().complete_cell();
+	}
 	inside = true;
 	rows.push_back(TableRow());
 }
 
 void HtmlRenderer::Table::add_text(const std::string& str)
 {
-	if (!inside)
+	if (!inside) {
 		start_row();
+	}
 	rows.back().add_text(str);
 }
 
@@ -948,18 +986,17 @@ void HtmlRenderer::render_table(const HtmlRenderer::Table& table,
 					table.rows[row].cells[cell].text.size();
 					idx++)
 					width = std::max(width,
-						utils::strwidth_stfl(
-							table.rows[row]
+							utils::strwidth_stfl(
+								table.rows[row]
 								.cells[cell]
 								.text[idx]));
 			}
 			if (table.rows[row].cells[cell].span > 1) {
 				width += table.rows[row].cells[cell].span;
+				// divide size evenly on columns (can be done better, I know)
 				width /= table.rows[row]
-						 .cells[cell]
-						 .span; // devide size evenly on
-							// columns (can be done
-							// better, I know)
+					.cells[cell]
+					.span;
 			}
 			cell_widths[cell] = std::max(cell_widths[cell], width);
 		}
@@ -971,15 +1008,17 @@ void HtmlRenderer::render_table(const HtmlRenderer::Table& table,
 
 	// create a row separator
 	std::string separator;
-	if (table.has_border)
+	if (table.has_border) {
 		separator += hvsep;
+	}
 	for (size_t cell = 0; cell < cells; cell++) {
 		separator += std::string(cell_widths[cell], hsep);
 		separator += hvsep;
 	}
 
-	if (!table.has_border)
+	if (!table.has_border) {
 		vsep = ' ';
+	}
 
 	// render the table
 	if (table.has_border)
@@ -991,65 +1030,68 @@ void HtmlRenderer::render_table(const HtmlRenderer::Table& table,
 		for (size_t cell = 0; cell < table.rows[row].cells.size();
 			cell++)
 			height = std::max(height,
-				table.rows[row].cells[cell].text.size());
+					table.rows[row].cells[cell].text.size());
 
 		for (size_t idx = 0; idx < height; ++idx) {
 			std::string line;
-			if (table.has_border)
+			if (table.has_border) {
 				line += vsep;
+			}
 			for (size_t cell = 0;
 				cell < table.rows[row].cells.size();
 				cell++) {
 				size_t cell_width = 0;
 				if (idx < table.rows[row]
-						  .cells[cell]
-						  .text.size()) {
+					.cells[cell]
+					.text.size()) {
 					LOG(Level::DEBUG,
-						"row = %d cell = %d text = %s",
-						row,
-						cell,
+						"row = %" PRIu64 " cell = %" PRIu64 " text = %s",
+						static_cast<uint64_t>(row),
+						static_cast<uint64_t>(cell),
 						table.rows[row]
-							.cells[cell]
-							.text[idx]);
+						.cells[cell]
+						.text[idx]);
 					cell_width = utils::strwidth_stfl(
-						table.rows[row]
+							table.rows[row]
 							.cells[cell]
 							.text[idx]);
 					line += table.rows[row]
-							.cells[cell]
-							.text[idx];
+						.cells[cell]
+						.text[idx];
 				}
 				size_t reference_width = cell_widths[cell];
 				if (table.rows[row].cells[cell].span > 1) {
 					for (size_t ic = cell + 1; ic < cell +
-							table.rows[row]
-								.cells[cell]
-								.span;
+						table.rows[row]
+						.cells[cell]
+						.span;
 						++ic)
 						reference_width +=
 							cell_widths[ic] + 1;
 				}
 				LOG(Level::DEBUG,
-					"cell_width = %d reference_width = %d",
-					cell_width,
-					reference_width);
+					"cell_width = %" PRIu64 " reference_width = %" PRIu64,
+					static_cast<uint64_t>(cell_width),
+					static_cast<uint64_t>(reference_width));
 				if (cell_width <
 					reference_width) // pad, if necessary
 					line += std::string(
-						reference_width - cell_width,
-						' ');
+							reference_width - cell_width,
+							' ');
 
-				if (cell < table.rows[row].cells.size() - 1)
+				if (cell < table.rows[row].cells.size() - 1) {
 					line += vsep;
+				}
 			}
-			if (table.has_border)
+			if (table.has_border) {
 				line += vsep;
+			}
 			lines.push_back(
 				std::make_pair(LineType::nonwrappable, line));
 		}
 		if (table.has_border)
 			lines.push_back(std::make_pair(
-				LineType::nonwrappable, separator));
+					LineType::nonwrappable, separator));
 	}
 }
 
@@ -1068,20 +1110,22 @@ std::string HtmlRenderer::get_char_numbering(unsigned int count)
 std::string HtmlRenderer::get_roman_numbering(unsigned int count)
 {
 	unsigned int values[] = {
-		1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1};
+		1000, 900, 500, 400, 100, 90, 50, 40, 10, 9, 5, 4, 1
+	};
 	const char* numerals[] = {"m",
-		"cm",
-		"d",
-		"cd",
-		"c",
-		"xc",
-		"l",
-		"xl",
-		"x",
-		"ix",
-		"v",
-		"iv",
-		"i"};
+			"cm",
+			"d",
+			"cd",
+			"c",
+			"xc",
+			"l",
+			"xl",
+			"x",
+			"ix",
+			"v",
+			"iv",
+			"i"
+		};
 	std::string result;
 	for (unsigned int i = 0; i < (sizeof(values) / sizeof(values[0]));
 		i++) {
