@@ -1,5 +1,6 @@
 #include "listformaction.h"
 
+#include "rssfeed.h"
 #include "view.h"
 
 namespace newsboat {
@@ -48,7 +49,7 @@ void ListFormAction::process_operation(Operation op,
 	}
 }
 
-void ListFormAction::open_unread_items_in_browser(std::shared_ptr<RssFeed> feed,
+int ListFormAction::open_unread_items_in_browser(std::shared_ptr<RssFeed> feed,
 	bool markread)
 {
 	int tabcount = 0;
@@ -56,7 +57,9 @@ void ListFormAction::open_unread_items_in_browser(std::shared_ptr<RssFeed> feed,
 		if (tabcount <
 			cfg->get_configvalue_as_int("max-browser-tabs")) {
 			if (item->unread()) {
-				v->open_in_browser(item->link());
+				if (int err = v->open_in_browser(item->link())) {
+					return err;
+				}
 				tabcount += 1;
 				item->set_unread(!markread);
 			}
@@ -64,6 +67,7 @@ void ListFormAction::open_unread_items_in_browser(std::shared_ptr<RssFeed> feed,
 			break;
 		}
 	}
+	return 0;
 }
 
 } // namespace newsboat
