@@ -30,7 +30,7 @@ void ListFormatter::set_line(const unsigned int itempos,
 
 	if (width > 0 && text.length() > 0) {
 		std::wstring mytext = utils::clean_nonprintable_characters(
-			utils::str2wstr(text));
+				utils::str2wstr(text));
 
 		while (mytext.length() > 0) {
 			size_t size = mytext.length();
@@ -38,19 +38,19 @@ void ListFormatter::set_line(const unsigned int itempos,
 			if (w > width) {
 				while (size &&
 					(w = utils::wcswidth_stfl(
-						 mytext, size)) > width) {
+								mytext, size)) > width) {
 					size--;
 				}
 			}
 			formatted_text.push_back(LineIdPair(
-				utils::wstr2str(mytext.substr(0, size)), id));
+					utils::wstr2str(mytext.substr(0, size)), id));
 			mytext.erase(0, size);
 		}
 	} else {
 		formatted_text.push_back(LineIdPair(
-			utils::wstr2str(utils::clean_nonprintable_characters(
-				utils::str2wstr(text))),
-			id));
+				utils::wstr2str(utils::clean_nonprintable_characters(
+						utils::str2wstr(text))),
+				id));
 	}
 
 	if (itempos == UINT_MAX) {
@@ -78,11 +78,12 @@ std::string ListFormatter::format_list(RegexManager* rxman,
 	format_cache = "{list";
 	for (const auto& line : lines) {
 		std::string str = line.first;
-		if (rxman)
+		if (rxman) {
 			rxman->quote_and_highlight(str, location);
+		}
 		if (line.second == UINT_MAX) {
 			format_cache.append(strprintf::fmt(
-				"{listitem text:%s}", Stfl::quote(str)));
+					"{listitem text:%s}", Stfl::quote(str)));
 		} else {
 			format_cache.append(
 				strprintf::fmt("{listitem[%u] text:%s}",
