@@ -1,14 +1,17 @@
 #include "matcher.h"
 
 #include <cassert>
+#include <cinttypes>
 #include <ctime>
 #include <regex.h>
 #include <sstream>
 #include <sys/time.h>
 #include <vector>
 
-#include "exceptions.h"
 #include "logger.h"
+#include "matchable.h"
+#include "matcherexception.h"
+#include "scopemeasure.h"
 #include "utils.h"
 
 namespace newsboat {
@@ -43,11 +46,11 @@ bool Matcher::parse(const std::string& expr)
 	}
 
 	gettimeofday(&tv2, nullptr);
-	unsigned long diff =
+	const uint64_t diff =
 		(((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) -
 		tv1.tv_usec;
 	LOG(Level::DEBUG,
-		"Matcher::parse: parsing `%s' took %lu µs (success = %d)",
+		"Matcher::parse: parsing `%s' took %" PRIu64 " µs (success = %d)",
 		expr,
 		diff,
 		b ? 1 : 0);
@@ -103,8 +106,9 @@ bool Matcher::matchop_between(expression* e, Matchable* item)
 	std::istringstream isatt(item->get_attribute(e->name));
 	int att;
 	isatt >> att;
-	if (lit.size() < 2)
+	if (lit.size() < 2) {
 		return false;
+	}
 	std::istringstream is1(lit[0]), is2(lit[1]);
 	int i1, i2;
 	is1 >> i1;
@@ -139,8 +143,10 @@ bool Matcher::matchop_rxeq(expression* e, Matchable* item)
 		e->regex = new regex_t;
 		int err;
 		if ((err = regcomp(e->regex,
-			     e->literal.c_str(),
-			     REG_EXTENDED | REG_ICASE | REG_NOSUB)) != 0) {
+						e->literal.c_str(),
+						REG_EXTENDED | REG_ICASE | REG_NOSUB)) != 0) {
+			delete e->regex;
+			e->regex = nullptr;
 			char buf[1024];
 			regerror(err, e->regex, buf, sizeof(buf));
 			throw MatcherException(
@@ -150,11 +156,12 @@ bool Matcher::matchop_rxeq(expression* e, Matchable* item)
 		}
 	}
 	if (regexec(e->regex,
-		    item->get_attribute(e->name).c_str(),
-		    0,
-		    nullptr,
-		    0) == 0)
+			item->get_attribute(e->name).c_str(),
+			0,
+			nullptr,
+			0) == 0) {
 		return true;
+	}
 	return false;
 }
 
@@ -193,12 +200,9 @@ bool Matcher::matches_r(expression* e, Matchable* item)
 		/* the operator "and" and "or" simply connect two different
 		 * subexpressions */
 		case LOGOP_AND:
+			// short-circuit evaluation in C -> short circuit evaluation in the filter language
 			return matches_r(e->l, item) &&
-				matches_r(e->r, item); // short-circuit
-						       // evaulation in C ->
-						       // short circuit
-						       // evaluation in the
-						       // filter language
+				matches_r(e->r, item);
 
 		case LOGOP_OR:
 			return matches_r(e->l, item) ||
