@@ -1,6 +1,7 @@
 #include "utils.h"
 
 #include <algorithm>
+#include <cinttypes>
 #include <cmath>
 #include <cstdarg>
 #include <cstdio>
@@ -12,7 +13,6 @@
 #include <fcntl.h>
 #include <iconv.h>
 #include <langinfo.h>
-#include <libgen.h>
 #include <libxml/uri.h>
 #include <locale>
 #include <mutex>
@@ -21,13 +21,11 @@
 #include <sstream>
 #include <stfl.h>
 #include <sys/param.h>
-#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/utsname.h>
 #include <unistd.h>
 #include <unordered_set>
 
-#include "3rd-party/alphanum.hpp"
 #include "config.h"
 #include "logger.h"
 #include "strprintf.h"
@@ -78,6 +76,11 @@ void append_escapes(std::string& str, char c)
 }
 }
 
+std::string utils::strip_comments(const std::string& line)
+{
+	return RustString(rs_strip_comments(line.c_str()));
+}
+
 std::vector<std::string> utils::tokenize_quoted(const std::string& str,
 	std::string delimiters)
 {
@@ -109,8 +112,9 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str,
 	std::string::size_type pos = last_pos;
 
 	while (pos != std::string::npos && last_pos != std::string::npos) {
-		if (str[last_pos] == '#') // stop as soon as we found a comment
+		if (str[last_pos] == '#') { // stop as soon as we found a comment
 			break;
+		}
 
 		if (str[last_pos] == '"') {
 			++last_pos;
@@ -236,15 +240,17 @@ std::vector<std::string> utils::tokenize_spaced(const std::string& str,
 	while (std::string::npos != pos || std::string::npos != last_pos) {
 		tokens.push_back(str.substr(last_pos, pos - last_pos));
 		last_pos = str.find_first_not_of(delimiters, pos);
-		if (last_pos > pos)
+		if (last_pos > pos) {
 			tokens.push_back(str.substr(pos, last_pos - pos));
+		}
 		pos = str.find_first_of(delimiters, last_pos);
 	}
 
 	return tokens;
 }
 
-std::string utils::consolidate_whitespace(const std::string& str) {
+std::string utils::consolidate_whitespace(const std::string& str)
+{
 
 	return RustString(rs_consolidate_whitespace(str.c_str()));
 }
@@ -257,7 +263,9 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str,
 	std::string::size_type pos = str.find_first_of(delimiters, last_pos);
 	unsigned int i;
 
-	LOG(Level::DEBUG, "utils::tokenize_nl: last_pos = %u", last_pos);
+	LOG(Level::DEBUG,
+		"utils::tokenize_nl: last_pos = %" PRIu64,
+		static_cast<uint64_t>(last_pos));
 	if (last_pos != std::string::npos) {
 		for (i = 0; i < last_pos; ++i) {
 			tokens.push_back(std::string("\n"));
@@ -271,8 +279,8 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str,
 			str.substr(last_pos, pos - last_pos));
 		last_pos = str.find_first_not_of(delimiters, pos);
 		LOG(Level::DEBUG,
-			"utils::tokenize_nl: pos - last_pos = %u",
-			last_pos - pos);
+			"utils::tokenize_nl: pos - last_pos = %" PRIu64,
+			static_cast<uint64_t>(last_pos - pos));
 		for (i = 0; last_pos != std::string::npos &&
 			pos != std::string::npos && i < (last_pos - pos);
 			++i) {
@@ -294,17 +302,18 @@ std::string utils::translit(const std::string& tocode,
 	static TranslitState state = TranslitState::UNKNOWN;
 
 	// TRANSLIT is not needed when converting to unicode encodings
-	if (tocode == "utf-8" || tocode == "WCHAR_T")
+	if (tocode == "utf-8" || tocode == "WCHAR_T") {
 		return tocode;
+	}
 
 	if (state == TranslitState::UNKNOWN) {
 		iconv_t cd = ::iconv_open(
-			(tocode + "//TRANSLIT").c_str(), fromcode.c_str());
+				(tocode + "//TRANSLIT").c_str(), fromcode.c_str());
 
 		if (cd == reinterpret_cast<iconv_t>(-1)) {
 			if (errno == EINVAL) {
 				iconv_t cd = ::iconv_open(
-					tocode.c_str(), fromcode.c_str());
+						tocode.c_str(), fromcode.c_str());
 				if (cd != reinterpret_cast<iconv_t>(-1)) {
 					state = TranslitState::UNSUPPORTED;
 				} else {
@@ -333,7 +342,7 @@ std::string utils::translit(const std::string& tocode,
 	}
 
 	return ((state == TranslitState::SUPPORTED) ? (tocode + tlit)
-						     : (tocode));
+			: (tocode));
 }
 
 std::string utils::convert_text(const std::string& text,
@@ -342,14 +351,16 @@ std::string utils::convert_text(const std::string& text,
 {
 	std::string result;
 
-	if (strcasecmp(tocode.c_str(), fromcode.c_str()) == 0)
+	if (strcasecmp(tocode.c_str(), fromcode.c_str()) == 0) {
 		return text;
+	}
 
 	iconv_t cd = ::iconv_open(
-		translit(tocode, fromcode).c_str(), fromcode.c_str());
+			translit(tocode, fromcode).c_str(), fromcode.c_str());
 
-	if (cd == reinterpret_cast<iconv_t>(-1))
+	if (cd == reinterpret_cast<iconv_t>(-1)) {
 		return result;
+	}
 
 	size_t inbytesleft;
 	size_t outbytesleft;
@@ -370,13 +381,13 @@ std::string utils::convert_text(const std::string& text,
 
 	outbytesleft = sizeof(outbuf);
 	inbufp = const_cast<char*>(
-		text.c_str()); // evil, but spares us some trouble
+			text.c_str()); // evil, but spares us some trouble
 	inbytesleft = strlen(inbufp);
 
 	do {
 		char* old_outbufp = outbufp;
 		int rc = ::iconv(
-			cd, &inbufp, &inbytesleft, &outbufp, &outbytesleft);
+				cd, &inbufp, &inbytesleft, &outbufp, &outbytesleft);
 		if (-1 == rc) {
 			switch (errno) {
 			case E2BIG:
@@ -429,8 +440,8 @@ void utils::extract_filter(const std::string& line,
 		url);
 }
 
-static size_t
-my_write_data(void* buffer, size_t size, size_t nmemb, void* userp)
+static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
+	void* userp)
 {
 	std::string* pbuf = static_cast<std::string*>(userp);
 	pbuf->append(static_cast<const char*>(buffer), size * nmemb);
@@ -503,6 +514,12 @@ std::string utils::resolve_tilde(const std::string& str)
 	return RustString(rs_resolve_tilde(str.c_str()));
 }
 
+std::string utils::resolve_relative(const std::string& reference,
+	const std::string& fname)
+{
+	return RustString(rs_resolve_relative(reference.c_str(), fname.c_str()));
+}
+
 std::string utils::replace_all(std::string str,
 	const std::string& from,
 	const std::string& to)
@@ -510,14 +527,6 @@ std::string utils::replace_all(std::string str,
 	return RustString( rs_replace_all(str.c_str(), from.c_str(), to.c_str()) );
 }
 
-std::wstring utils::utf8str2wstr(const std::string& utf8str)
-{
-	stfl_ipool* pool = stfl_ipool_create("utf-8");
-	std::wstring wstr = stfl_ipool_towc(pool, utf8str.c_str());
-	stfl_ipool_destroy(pool);
-	return wstr;
-}
-
 std::wstring utils::str2wstr(const std::string& str)
 {
 	const char* codeset = nl_langinfo(CODESET);
@@ -557,15 +566,15 @@ std::string utils::get_useragent(ConfigContainer* cfgcont)
 				PROCESSOR = "Intel ";
 			}
 			return strprintf::fmt("%s/%s (Macintosh; %sMac OS X)",
-				PROGRAM_NAME,
-				PROGRAM_VERSION,
-				PROCESSOR);
+					PROGRAM_NAME,
+					utils::program_version(),
+					PROCESSOR);
 		}
 		return strprintf::fmt("%s/%s (%s %s)",
-			PROGRAM_NAME,
-			PROGRAM_VERSION,
-			buf.sysname,
-			buf.machine);
+				PROGRAM_NAME,
+				utils::program_version(),
+				buf.sysname,
+				buf.machine);
 	}
 	return ua_pref;
 }
@@ -576,42 +585,6 @@ unsigned int utils::to_u(const std::string& str,
 	return rs_to_u(str.c_str(), default_value);
 }
 
-ScopeMeasure::ScopeMeasure(const std::string& func, Level ll)
-	: funcname(func)
-	, lvl(ll)
-{
-	gettimeofday(&tv1, nullptr);
-}
-
-void ScopeMeasure::stopover(const std::string& son)
-{
-	gettimeofday(&tv2, nullptr);
-	unsigned long diff =
-		(((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) -
-		tv1.tv_usec;
-	LOG(lvl,
-		"ScopeMeasure: function `%s' (stop over `%s') took %lu.%06lu "
-		"s so "
-		"far",
-		funcname,
-		son,
-		diff / 1000000,
-		diff % 1000000);
-}
-
-ScopeMeasure::~ScopeMeasure()
-{
-	gettimeofday(&tv2, nullptr);
-	unsigned long diff =
-		(((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) -
-		tv1.tv_usec;
-	LOG(Level::INFO,
-		"ScopeMeasure: function `%s' took %lu.%06lu s",
-		funcname,
-		diff / 1000000,
-		diff % 1000000);
-}
-
 bool utils::is_valid_color(const std::string& color)
 {
 	return rs_is_valid_color(color.c_str());
@@ -623,9 +596,9 @@ bool utils::is_valid_attribute(const std::string& attrib)
 }
 
 std::vector<std::pair<unsigned int, unsigned int>> utils::partition_indexes(
-	unsigned int start,
-	unsigned int end,
-	unsigned int parts)
+		unsigned int start,
+		unsigned int end,
+		unsigned int parts)
 {
 	std::vector<std::pair<unsigned int, unsigned int>> partitions;
 	unsigned int count = end - start + 1;
@@ -633,7 +606,7 @@ std::vector<std::pair<unsigned int, unsigned int>> utils::partition_indexes(
 
 	for (unsigned int i = 0; i < parts - 1; i++) {
 		partitions.push_back(std::pair<unsigned int, unsigned int>(
-			start, start + size - 1));
+				start, start + size - 1));
 		start += size;
 	}
 
@@ -781,14 +754,7 @@ std::string utils::censor_url(const std::string& url)
 
 std::string utils::quote_for_stfl(std::string str)
 {
-	unsigned int len = str.length();
-	for (unsigned int i = 0; i < len; ++i) {
-		if (str[i] == '<') {
-			str.insert(i + 1, ">");
-			++len;
-		}
-	}
-	return str;
+	return RustString(rs_quote_for_stfl(str.c_str()));
 }
 
 void utils::trim(std::string& str)
@@ -906,68 +872,32 @@ std::string utils::get_content(xmlNode* node)
 
 std::string utils::get_basename(const std::string& url)
 {
-	std::string retval;
-	xmlURIPtr uri = xmlParseURI(url.c_str());
-	if (uri && uri->path) {
-		std::string path(uri->path);
-		// check for path ending with an empty filename
-		if (path[path.length() - 1] != '/') {
-			char* base = basename(uri->path);
-			if (base) {
-				// check for empty path
-				if (base[0] != '/') {
-					retval = std::string(base);
-				}
-			}
-		}
-		xmlFreeURI(uri);
-	}
-	return retval;
+	return RustString(rs_get_basename(url.c_str()));
 }
 
 unsigned long utils::get_auth_method(const std::string& type)
 {
-	if (type == "any")
-		return CURLAUTH_ANY;
-	if (type == "basic")
-		return CURLAUTH_BASIC;
-	if (type == "digest")
-		return CURLAUTH_DIGEST;
-#ifdef CURLAUTH_DIGEST_IE
-	if (type == "digest_ie")
-		return CURLAUTH_DIGEST_IE;
-#else
-#warning \
-	"proxy-auth-method digest_ie not added due to libcurl older than 7.19.3"
-#endif
-	if (type == "gssnegotiate")
-		return CURLAUTH_GSSNEGOTIATE;
-	if (type == "ntlm")
-		return CURLAUTH_NTLM;
-	if (type == "anysafe")
-		return CURLAUTH_ANYSAFE;
-	if (type != "") {
-		LOG(Level::USERERROR,
-			"you configured an invalid proxy authentication "
-			"method: %s",
-			type);
-	}
-	return CURLAUTH_ANY;
+	return rs_get_auth_method(type.c_str());
 }
 
 curl_proxytype utils::get_proxy_type(const std::string& type)
 {
-	if (type == "http")
+	if (type == "http") {
 		return CURLPROXY_HTTP;
-	if (type == "socks4")
+	}
+	if (type == "socks4") {
 		return CURLPROXY_SOCKS4;
-	if (type == "socks5")
+	}
+	if (type == "socks5") {
 		return CURLPROXY_SOCKS5;
-	if (type == "socks5h")
+	}
+	if (type == "socks5h") {
 		return CURLPROXY_SOCKS5_HOSTNAME;
+	}
 #ifdef CURLPROXY_SOCKS4A
-	if (type == "socks4a")
+	if (type == "socks4a") {
 		return CURLPROXY_SOCKS4A;
+	}
 #endif
 
 	if (type != "") {
@@ -992,60 +922,23 @@ std::string utils::unescape_url(const std::string& url)
 std::wstring utils::clean_nonprintable_characters(std::wstring text)
 {
 	for (size_t idx = 0; idx < text.size(); ++idx) {
-		if (!iswprint(text[idx]))
+		if (!iswprint(text[idx])) {
 			text[idx] = L'\uFFFD';
+		}
 	}
 	return text;
 }
 
 unsigned int utils::gentabs(const std::string& str)
 {
-	int tabcount = 4 - (utils::strwidth(str) / 8);
-	if (tabcount <= 0) {
-		tabcount = 1;
-	}
-	return tabcount;
+	return rs_gentabs(str.c_str());
 }
 
 /* Like mkdir(), but creates ancestors (parent directories) if they don't
  * exist. */
 int utils::mkdir_parents(const std::string& p, mode_t mode)
 {
-	int result = -1;
-
-	/* Have to copy the path because we're going to modify it */
-	const size_t length = p.length() + 1;
-	char* pathname = (char*)malloc(length);
-	if (pathname == nullptr) {
-		// errno is already set by malloc(), so simply return with an error
-		return -1;
-	}
-	strncpy(pathname, p.c_str(), length);
-	/* This pointer will run through the whole string looking for '/'.
-	 * We move it by one if path starts with slash because if we don't, the
-	 * first call to access() will fail (because of empty path) */
-	char* curr = pathname + (*pathname == '/' ? 1 : 0);
-
-	while (*curr) {
-		if (*curr == '/') {
-			*curr = '\0';
-			result = access(pathname, F_OK);
-			if (result == -1) {
-				result = mkdir(pathname, mode);
-				if (result != 0 && errno != EEXIST)
-					break;
-			}
-			*curr = '/';
-		}
-		curr++;
-	}
-
-	if (result == 0) {
-		result = mkdir(p.c_str(), mode);
-	}
-
-	free(pathname);
-	return result;
+	return rs_mkdir_parents(p.c_str(), static_cast<std::uint32_t>(mode));
 }
 
 std::string utils::make_title(const std::string& const_url)
@@ -1073,25 +966,12 @@ int utils::run_interactively(const std::string& command,
 
 std::string utils::getcwd()
 {
-	// Linux seem to have its MAX_PATH set somewhere around this value, so
-	// should be a nice default
-	std::vector<char> result(4096, '\0');
-
-	while (true) {
-		char* ret = ::getcwd(result.data(), result.size());
-		if (ret == nullptr && errno == ERANGE) {
-			result.resize(result.size() * 2);
-		} else {
-			break;
-		}
-	}
-
-	return std::string(result.data());
+	return RustString(rs_getcwd());
 }
 
 int utils::strnaturalcmp(const std::string& a, const std::string& b)
 {
-	return doj::alphanum_comp(a, b);
+	return rs_strnaturalcmp(a.c_str(), b.c_str());
 }
 
 void utils::remove_soft_hyphens(std::string& text)
@@ -1099,12 +979,7 @@ void utils::remove_soft_hyphens(std::string& text)
 	/* Remove all soft-hyphens as they can behave unpredictably (see
 	 * https://github.com/akrennmair/newsbeuter/issues/259#issuecomment-259609490)
 	 * and inadvertently render as hyphens */
-
-	std::string::size_type pos = text.find("\u00AD");
-	while (pos != std::string::npos) {
-		text.erase(pos, 2);
-		pos = text.find("\u00AD", pos);
-	}
+	text = RustString(rs_remove_soft_hyphens(text.c_str()));
 }
 
 bool utils::is_valid_podcast_type(const std::string& mimetype)
@@ -1112,21 +987,21 @@ bool utils::is_valid_podcast_type(const std::string& mimetype)
 	return rs_is_valid_podcast_type(mimetype.c_str());
 }
 
-	/*
-	 * See
-	 * http://curl.haxx.se/libcurl/c/libcurl-tutorial.html#Multi-threading
-	 * for a reason why we do this.
-	 *
-	 * These callbacks are deprecated as of OpenSSL 1.1.0; see the
-	 * changelog: https://www.openssl.org/news/changelog.html#x6
-	 */
+/*
+ * See
+ * http://curl.haxx.se/libcurl/c/libcurl-tutorial.html#Multi-threading
+ * for a reason why we do this.
+ *
+ * These callbacks are deprecated as of OpenSSL 1.1.0; see the
+ * changelog: https://www.openssl.org/news/changelog.html#x6
+ */
 
 #if HAVE_OPENSSL && OPENSSL_VERSION_NUMBER < 0x01010000fL
 static std::mutex* openssl_mutexes = nullptr;
 static int openssl_mutexes_size = 0;
 
-static void
-openssl_mth_locking_function(int mode, int n, const char* file, int line)
+static void openssl_mth_locking_function(int mode, int n, const char* file,
+	int line)
 {
 	if (n < 0 || n >= openssl_mutexes_size) {
 		LOG(Level::ERROR,
@@ -1171,4 +1046,34 @@ std::string utils::get_default_browser()
 	return RustString(rs_get_default_browser());
 }
 
+std::string utils::program_version()
+{
+	return RustString(rs_program_version());
+}
+
+unsigned int utils::newsboat_version_major()
+{
+	return rs_newsboat_version_major();
+}
+
+std::string utils::mt_strf_localtime(const std::string& format, time_t t)
+{
+	// localtime() returns a pointer to static memory, so we need to protect
+	// its caller with a mutex to ensure that no two threads concurrently
+	// access that static memory. In Newsboat, the only caller for localtime()
+	// is strftime(), that's why this function bakes the two together.
+
+	static std::mutex mtx;
+	std::lock_guard<std::mutex> guard(mtx);
+
+	const size_t BUFFER_SIZE = 4096;
+	char buffer[BUFFER_SIZE];
+	const size_t written = strftime(buffer,
+			BUFFER_SIZE,
+			format.c_str(),
+			localtime(&t));
+
+	return std::string(buffer, written);
+}
+
 } // namespace newsboat
