@@ -24,15 +24,15 @@ extern "C" {
 }
 
 #include "config.h"
+#include "dbexception.h"
 #include "dialogs.h"
 #include "dialogsformaction.h"
 #include "exception.h"
-#include "exceptions.h"
 #include "feedlist.h"
 #include "feedlistformaction.h"
 #include "filebrowser.h"
+#include "fmtstrformatter.h"
 #include "formaction.h"
-#include "formatstring.h"
 #include "help.h"
 #include "helpformaction.h"
 #include "htmlrenderer.h"
@@ -42,9 +42,10 @@ extern "C" {
 #include "itemviewformaction.h"
 #include "keymap.h"
 #include "logger.h"
+#include "matcherexception.h"
 #include "regexmanager.h"
 #include "reloadthread.h"
-#include "rss.h"
+#include "rssfeed.h"
 #include "selectformaction.h"
 #include "selecttag.h"
 #include "strprintf.h"
@@ -177,7 +178,7 @@ int View::run()
 
 	// create feedlist
 	auto feedlist = std::make_shared<FeedListFormAction>(
-		this, feedlist_str, rsscache, filters, cfg);
+			this, feedlist_str, rsscache, filters, cfg);
 	set_bindings(feedlist);
 	feedlist->set_regexmanager(rxman);
 	feedlist->set_tags(tags);
@@ -210,12 +211,8 @@ int View::run()
 			fa->process_op(
 				macrocmds[0].op, true, &macrocmds[0].args);
 
-			macrocmds.erase(macrocmds.begin()); // remove first
-							    // macro command,
-							    // since it has
-							    // already been
-							    // processed
-
+			// remove first macro command, since it has already been processed
+			macrocmds.erase(macrocmds.begin());
 		} else {
 			// we then receive the event and ignore timeouts.
 			const char* event = fa->get_form()->run(60000);
@@ -224,9 +221,9 @@ int View::run()
 				ctrl_c_hit = false;
 				cancel_input(fa);
 				if (!get_cfg()->get_configvalue_as_bool(
-					    "confirm-exit") ||
+						"confirm-exit") ||
 					confirm(_("Do you really want to quit "
-						  "(y:Yes n:No)? "),
+							"(y:Yes n:No)? "),
 						_("yn")) == *_("y")) {
 					Stfl::reset();
 					return EXIT_FAILURE;
@@ -236,9 +233,9 @@ int View::run()
 			if (!event || strcmp(event, "TIMEOUT") == 0) {
 				if (fa->id() == "article")
 					std::dynamic_pointer_cast<
-						ItemViewFormAction,
-						FormAction>(fa)
-						->update_percent();
+					ItemViewFormAction,
+					FormAction>(fa)
+					->update_percent();
 				continue;
 			}
 
@@ -317,8 +314,9 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 
 		const char* event = fa->get_form()->run(1000);
 		LOG(Level::DEBUG, "View::run: event = %s", event);
-		if (!event || strcmp(event, "TIMEOUT") == 0)
+		if (!event || strcmp(event, "TIMEOUT") == 0) {
 			continue;
+		}
 
 		Operation op = keys->get_operation(event, fa->id());
 
@@ -330,10 +328,11 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 		fa->process_op(op);
 	}
 
-	if (value == "")
+	if (value == "") {
 		return "";
-	else
+	} else {
 		return f->get_value(value);
+	}
 }
 
 std::string View::get_filename_suggestion(const std::string& s)
@@ -344,16 +343,18 @@ std::string View::get_filename_suggestion(const std::string& s)
 	 */
 	std::string retval;
 	for (unsigned int i = 0; i < s.length(); ++i) {
-		if (isalnum(s[i]))
+		if (isalnum(s[i])) {
 			retval.append(1, s[i]);
-		else if (s[i] == '/' || s[i] == ' ' || s[i] == '\r' ||
-			s[i] == '\n')
+		} else if (s[i] == '/' || s[i] == ' ' || s[i] == '\r' ||
+			s[i] == '\n') {
 			retval.append(1, '_');
+		}
 	}
-	if (retval.length() == 0)
+	if (retval.length() == 0) {
 		retval = "article.txt";
-	else
+	} else {
 		retval.append(".txt");
+	}
 	LOG(Level::DEBUG, "View::get_filename_suggestion: %s -> %s", s, retval);
 	return retval;
 }
@@ -376,12 +377,13 @@ void View::open_in_pager(const std::string& filename)
 		cmdline = fmt.do_format(pager, 0);
 	} else {
 		const char* env_pager = nullptr;
-		if (pager != "")
+		if (pager != "") {
 			cmdline.append(pager);
-		else if ((env_pager = getenv("PAGER")) != nullptr)
+		} else if ((env_pager = getenv("PAGER")) != nullptr) {
 			cmdline.append(env_pager);
-		else
+		} else {
 			cmdline.append("more");
+		}
 		cmdline.append(" ");
 		cmdline.append(filename);
 	}
@@ -390,7 +392,7 @@ void View::open_in_pager(const std::string& filename)
 	pop_current_formaction();
 }
 
-void View::open_in_browser(const std::string& url)
+int View::open_in_browser(const std::string& url)
 {
 	formaction_stack.push_back(std::shared_ptr<FormAction>());
 	current_formaction = formaction_stack_size() - 1;
@@ -405,17 +407,19 @@ void View::open_in_browser(const std::string& url)
 		fmt.register_fmt('u', newurl);
 		cmdline = fmt.do_format(browser, 0);
 	} else {
-		if (browser != "")
+		if (browser != "") {
 			cmdline.append(browser);
-		else
+		} else {
 			cmdline.append("lynx");
+		}
 		cmdline.append(" '");
 		cmdline.append(utils::replace_all(url, "'", "%27"));
 		cmdline.append("'");
 	}
 	Stfl::reset();
-	utils::run_interactively(cmdline, "View::open_in_browser");
+	int ret = utils::run_interactively(cmdline, "View::open_in_browser");
 	pop_current_formaction();
+	return ret;
 }
 
 void View::update_visible_feeds(std::vector<std::shared_ptr<RssFeed>> feeds)
@@ -425,12 +429,12 @@ void View::update_visible_feeds(std::vector<std::shared_ptr<RssFeed>> feeds)
 			std::lock_guard<std::mutex> lock(mtx);
 			std::shared_ptr<FeedListFormAction> feedlist =
 				std::dynamic_pointer_cast<FeedListFormAction,
-					FormAction>(formaction_stack[0]);
+				FormAction>(formaction_stack[0]);
 			feedlist->update_visible_feeds(feeds);
 		}
 	} catch (const MatcherException& e) {
 		set_status(strprintf::fmt(
-			_("Error: applying the filter failed: %s"), e.what()));
+				_("Error: applying the filter failed: %s"), e.what()));
 		LOG(Level::DEBUG,
 			"View::update_visible_feeds: inside catch: %s",
 			e.what());
@@ -451,12 +455,12 @@ void View::set_feedlist(std::vector<std::shared_ptr<RssFeed>> feeds)
 		if (formaction_stack_size() > 0) {
 			std::shared_ptr<FeedListFormAction> feedlist =
 				std::dynamic_pointer_cast<FeedListFormAction,
-					FormAction>(formaction_stack[0]);
+				FormAction>(formaction_stack[0]);
 			feedlist->set_feedlist(feeds);
 		}
 	} catch (const MatcherException& e) {
 		set_status(strprintf::fmt(
-			_("Error: applying the filter failed: %s"), e.what()));
+				_("Error: applying the filter failed: %s"), e.what()));
 	}
 }
 
@@ -527,7 +531,7 @@ void View::push_itemlist(unsigned int pos)
 	if (feed->total_item_count() > 0) {
 		std::shared_ptr<ItemListFormAction> itemlist =
 			std::dynamic_pointer_cast<ItemListFormAction,
-				FormAction>(get_current_formaction());
+			FormAction>(get_current_formaction());
 		itemlist->set_pos(pos);
 	}
 }
@@ -541,7 +545,7 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 
 		std::shared_ptr<ItemListFormAction> itemlist =
 			std::dynamic_pointer_cast<ItemListFormAction,
-				FormAction>(fa);
+			FormAction>(fa);
 		assert(itemlist != nullptr);
 		std::shared_ptr<ItemViewFormAction> itemview(
 			new ItemViewFormAction(
@@ -551,8 +555,9 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 		itemview->set_feed(f);
 		itemview->set_guid(guid);
 		itemview->set_parent_formaction(fa);
-		if (searchphrase.length() > 0)
+		if (searchphrase.length() > 0) {
 			itemview->set_highlightphrase(searchphrase);
+		}
 		apply_colors(itemview);
 		itemview->init();
 		formaction_stack.push_back(itemview);
@@ -570,8 +575,8 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 			}
 		} catch (const DbException& e) {
 			show_error(strprintf::fmt(
-				_("Error while marking article as read: %s"),
-				e.what()));
+					_("Error while marking article as read: %s"),
+					e.what()));
 		}
 		::unlink(filename.c_str());
 	}
@@ -634,6 +639,17 @@ std::string View::run_filebrowser(const std::string& default_filename,
 	return run_modal(filebrowser, "filenametext");
 }
 
+std::string View::run_dirbrowser(const std::string& dir)
+{
+	std::shared_ptr<DirBrowserFormAction> dirbrowser(
+		new DirBrowserFormAction(this, filebrowser_str, cfg));
+	set_bindings(dirbrowser);
+	apply_colors(dirbrowser);
+	dirbrowser->set_dir(dir);
+	dirbrowser->set_parent_formaction(get_current_formaction());
+	return run_modal(dirbrowser, "filenametext");
+}
+
 std::string View::select_tag()
 {
 	if (tags.size() == 0) {
@@ -678,8 +694,9 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 	do {
 		const char* event = f->get_form()->run(0);
 		LOG(Level::DEBUG, "View::confirm: event = %s", event);
-		if (!event)
+		if (!event) {
 			continue;
+		}
 		if (strcmp(event, "ESC") == 0 || strcmp(event, "ENTER") == 0) {
 			result = 0;
 			LOG(Level::DEBUG,
@@ -708,7 +725,7 @@ void View::notify_itemlist_change(std::shared_ptr<RssFeed> feed)
 		if (form != nullptr && form->id() == "articlelist") {
 			std::shared_ptr<ItemListFormAction> itemlist =
 				std::dynamic_pointer_cast<ItemListFormAction,
-					FormAction>(form);
+				FormAction>(form);
 			if (itemlist != nullptr) {
 				std::shared_ptr<RssFeed> f =
 					itemlist->get_feed();
@@ -1006,15 +1023,14 @@ void View::pop_current_formaction()
 {
 	std::shared_ptr<FormAction> f = get_current_formaction();
 	auto it = formaction_stack.begin();
-	for (unsigned int i = 0; i < current_formaction; i++)
+	for (unsigned int i = 0; i < current_formaction; i++) {
 		++it;
+	}
 	formaction_stack.erase(it);
 	if (f == nullptr) {
+		// XXX TODO this is not correct... we'd need to return to the previous one, but nullptr formactions have no parent
 		current_formaction = formaction_stack_size() -
-			1; // XXX TODO this is not correct...
-			   // we'd need to return to the previous
-			   // one, but nullptr formactions have
-			   // no parent
+			1;
 	} else if (formaction_stack.size() > 0) {
 		// first, we set back the parent formactions of those who
 		// reference the formaction we just removed
@@ -1053,8 +1069,9 @@ void View::remove_formaction(unsigned int pos)
 {
 	std::shared_ptr<FormAction> f = formaction_stack[pos];
 	auto it = formaction_stack.begin();
-	for (unsigned int i = 0; i < pos; i++)
+	for (unsigned int i = 0; i < pos; i++) {
 		++it;
+	}
 	formaction_stack.erase(it);
 	current_formaction--;
 	if (f != nullptr && formaction_stack.size() > 0) {
@@ -1106,14 +1123,16 @@ void View::apply_colors(std::shared_ptr<FormAction> fa)
 			colorattr.append(fgcit->second);
 		}
 		if (bgcit->second != "default") {
-			if (colorattr.length() > 0)
+			if (colorattr.length() > 0) {
 				colorattr.append(",");
+			}
 			colorattr.append("bg=");
 			colorattr.append(bgcit->second);
 		}
 		for (const auto& attr : attit->second) {
-			if (colorattr.length() > 0)
+			if (colorattr.length() > 0) {
 				colorattr.append(",");
+			}
 			colorattr.append("attr=");
 			colorattr.append(attr);
 		}
@@ -1123,10 +1142,12 @@ void View::apply_colors(std::shared_ptr<FormAction> fa)
 			if (fa->id() == "article") {
 				std::string bold = article_colorstr;
 				std::string ul = article_colorstr;
-				if (bold.length() > 0)
+				if (bold.length() > 0) {
 					bold.append(",");
-				if (ul.length() > 0)
+				}
+				if (ul.length() > 0) {
 					ul.append(",");
+				}
 				bold.append("attr=bold");
 				ul.append("attr=underline");
 				fa->get_form()->set("color_bold", bold.c_str());
@@ -1150,8 +1171,9 @@ void View::apply_colors(std::shared_ptr<FormAction> fa)
 					styleend_str.append("bg=");
 					styleend_str.append(bgcit->second);
 				}
-				if (styleend_str.length() > 0)
+				if (styleend_str.length() > 0) {
 					styleend_str.append(",");
+				}
 				styleend_str.append("attr=bold");
 
 				fa->get_form()->set(
@@ -1166,7 +1188,7 @@ void View::feedlist_mark_pos_if_visible(unsigned int pos)
 	if (formaction_stack_size() > 0) {
 		std::dynamic_pointer_cast<FeedListFormAction, FormAction>(
 			formaction_stack[0])
-			->mark_pos_if_visible(pos);
+		->mark_pos_if_visible(pos);
 	}
 }
 
@@ -1203,8 +1225,9 @@ std::vector<std::pair<unsigned int, std::string>> View::get_formaction_names()
 void View::goto_next_dialog()
 {
 	current_formaction++;
-	if (current_formaction >= formaction_stack.size())
+	if (current_formaction >= formaction_stack.size()) {
 		current_formaction = 0;
+	}
 }
 
 void View::goto_prev_dialog()
@@ -1258,8 +1281,9 @@ void View::delete_word(std::shared_ptr<FormAction> fa)
 	std::string::size_type firstpos = curpos;
 	LOG(Level::DEBUG, "View::delete_word: before val = %s", val);
 	if (firstpos >= val.length() || ::isspace(val[firstpos])) {
-		if (firstpos != 0 && firstpos >= val.length())
+		if (firstpos != 0 && firstpos >= val.length()) {
 			firstpos = val.length() - 1;
+		}
 		while (firstpos > 0 && ::isspace(val[firstpos])) {
 			--firstpos;
 		}
@@ -1267,8 +1291,9 @@ void View::delete_word(std::shared_ptr<FormAction> fa)
 	while (firstpos > 0 && !::isspace(val[firstpos])) {
 		--firstpos;
 	}
-	if (firstpos != 0)
+	if (firstpos != 0) {
 		firstpos++;
+	}
 	val.erase(firstpos, curpos - firstpos);
 	LOG(Level::DEBUG, "View::delete_word: after val = %s", val);
 	fa->get_form()->set("qna_value", val);
@@ -1292,8 +1317,8 @@ void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 			"for "
 			"`%s'",
 			fragment);
-		::beep(); // direct call to ncurses - we beep to signal that
-			  // there is no suggestion available, just like vim
+		// direct call to ncurses - we beep to signal that there is no suggestion available, just like vim
+		::beep();
 		return;
 	case 1:
 		suggestion = suggestions[0];
@@ -1313,15 +1338,15 @@ void View::dump_current_form()
 	std::string formtext =
 		formaction_stack[current_formaction]->get_form()->dump(
 			"", "", 0);
-	char fnbuf[128];
 	time_t t = time(nullptr);
-	struct tm* stm = localtime(&t);
-	strftime(fnbuf, sizeof(fnbuf), "dumpform-%Y%m%d-%H%M%S.stfl", stm);
+	const auto fnbuf = utils::mt_strf_localtime(
+			"dumpform-%Y%m%d-%H%M%S.stfl",
+			t);
 	std::fstream f(fnbuf, std::ios_base::out);
 	if (!f.is_open()) {
 		show_error(strprintf::fmt("Error: couldn't open file %s: %s",
-			fnbuf,
-			strerror(errno)));
+				fnbuf,
+				strerror(errno)));
 		return;
 	}
 	f << formtext;
