@@ -9,7 +9,7 @@
 using namespace newsboat;
 
 TEST_CASE("URL reader remembers the file name from which it read the URLs",
-		"[FileUrlReader]")
+	"[FileUrlReader]")
 {
 	const std::string url("data/test-urls.txt");
 
@@ -52,14 +52,14 @@ TEST_CASE("URL reader keeps track of unique tags", "[FileUrlReader]")
 }
 
 TEST_CASE("URL reader writes files that it can understand later",
-		"[FileUrlReader]")
+	"[FileUrlReader]")
 {
 	const std::string testDataPath("data/test-urls.txt");
 
 	TestHelpers::TempFile urlsFile;
 
 	std::ofstream urlsFileStream;
-	urlsFileStream.open(urlsFile.getPath());
+	urlsFileStream.open(urlsFile.get_path());
 	REQUIRE(urlsFileStream.is_open());
 
 	std::ifstream testData;
@@ -73,19 +73,19 @@ TEST_CASE("URL reader writes files that it can understand later",
 	urlsFileStream.close();
 	testData.close();
 
-	FileUrlReader u(urlsFile.getPath());
+	FileUrlReader u(urlsFile.get_path());
 	u.reload();
 	REQUIRE_FALSE(u.get_urls().empty());
 	REQUIRE_FALSE(u.get_alltags().empty());
 
-	urlsFileStream.open(urlsFile.getPath());
+	urlsFileStream.open(urlsFile.get_path());
 	REQUIRE(urlsFileStream.is_open());
 	urlsFileStream << std::string();
 	urlsFileStream.close();
 
 	u.write_config();
 
-	FileUrlReader u2(urlsFile.getPath());
+	FileUrlReader u2(urlsFile.get_path());
 	u2.reload();
 	REQUIRE_FALSE(u2.get_urls().empty());
 	REQUIRE_FALSE(u2.get_alltags().empty());
