@@ -8,6 +8,7 @@
 #include "itemlist.h"
 #include "keymap.h"
 #include "regexmanager.h"
+#include "rssfeed.h"
 #include "test-helpers.h"
 
 using namespace newsboat;
@@ -19,7 +20,7 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 	newsboat::View v(&c);
 	TestHelpers::TempFile pagerfile;
 
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string test_title = "Article Title";
 	std::string test_author = "Article Author";
 	std::string test_description = "Article Description";
@@ -34,7 +35,7 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 	FilterContainer filters;
 
 	Cache rsscache(":memory:", &cfg);
-	cfg.set_configvalue("pager", "cat %f > " + pagerfile.getPath());
+	cfg.set_configvalue("pager", "cat %f > " + pagerfile.get_path());
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 
@@ -55,7 +56,7 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 
 	REQUIRE_NOTHROW(itemlist.process_op(OP_OPEN));
 
-	TestHelpers::AssertArticleFileContent(pagerfile.getPath(),
+	TestHelpers::AssertArticleFileContent(pagerfile.get_path(),
 		test_title,
 		test_author,
 		test_pubDate_str,
@@ -81,14 +82,12 @@ TEST_CASE("OP_PURGE_DELETED purges previously deleted items",
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
 	itemlist.set_feed(feed);
 
-	SECTION("No items to purge")
-	{
+	SECTION("No items to purge") {
 		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED));
 		REQUIRE(feed->total_item_count() == 1);
 	}
 
-	SECTION("Deleted items are purged")
-	{
+	SECTION("Deleted items are purged") {
 		item->set_deleted(true);
 		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED));
 		REQUIRE(feed->total_item_count() == 0);
@@ -103,11 +102,11 @@ TEST_CASE(
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
 
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string line;
 
 	ConfigContainer cfg;
-	cfg.set_configvalue("browser", "echo %u >> " + browserfile.getPath());
+	cfg.set_configvalue("browser", "echo %u >> " + browserfile.get_path());
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
@@ -124,7 +123,7 @@ TEST_CASE(
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
 	itemlist.set_feed(feed);
 	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
-	std::ifstream browserFileStream(browserfile.getPath());
+	std::ifstream browserFileStream(browserfile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
 	REQUIRE(line == test_url);
@@ -132,17 +131,48 @@ TEST_CASE(
 	REQUIRE(feed->unread_item_count() == 0);
 }
 
+TEST_CASE(
+	"OP_OPENBROWSER_AND_MARK does not mark read when browser fails",
+	"[ItemListFormAction]")
+{
+	Controller c;
+	newsboat::View v(&c);
+
+	const std::string test_url = "http://test_url";
+
+	ConfigContainer cfg;
+	cfg.set_configvalue("browser", "false %u");
+
+	Cache rsscache(":memory:", &cfg);
+	FilterContainer filters;
+
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
+	item->set_link(test_url);
+	item->set_unread(true);
+	feed->add_item(item);
+
+	v.set_config_container(&cfg);
+	c.set_view(&v);
+
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	itemlist.set_feed(feed);
+	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
+
+	REQUIRE(feed->unread_item_count() == 1);
+}
+
 TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 	"[ItemListFormAction]")
 {
 	Controller c;
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string line;
 
 	ConfigContainer cfg;
-	cfg.set_configvalue("browser", "echo %u >> " + browserfile.getPath());
+	cfg.set_configvalue("browser", "echo %u >> " + browserfile.get_path());
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
@@ -158,7 +188,7 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
 	itemlist.set_feed(feed);
 	itemlist.process_op(OP_OPENINBROWSER);
-	std::ifstream browserFileStream(browserfile.getPath());
+	std::ifstream browserFileStream(browserfile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
 	REQUIRE(line == test_url);
@@ -171,12 +201,12 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
 	std::unordered_set<std::string> url_set;
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string line;
 	int itemCount = 6;
 
 	ConfigContainer cfg;
-	cfg.set_configvalue("browser", "echo %u >> " + browserfile.getPath());
+	cfg.set_configvalue("browser", "echo %u >> " + browserfile.get_path());
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
@@ -198,8 +228,7 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
 	itemlist.set_feed(feed);
 
-	SECTION("unread >= max-browser-tabs")
-	{
+	SECTION("unread >= max-browser-tabs") {
 		int maxItemsToOpen = 4;
 		int openedItemsCount = 0;
 		cfg.set_configvalue(
@@ -207,13 +236,13 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 
 		itemlist.process_op(OP_OPENALLUNREADINBROWSER);
 
-		std::ifstream browserFileStream(browserfile.getPath());
+		std::ifstream browserFileStream(browserfile.get_path());
 		openedItemsCount = 0;
 		if (browserFileStream.is_open()) {
 			while (std::getline(browserFileStream, line)) {
 				INFO("Each URL should be present exactly once. "
-				     "Erase urls after first match to fail if "
-				     "an item opens twice.")
+					"Erase urls after first match to fail if "
+					"an item opens twice.")
 				REQUIRE(url_set.count(line) == 1);
 				url_set.erase(url_set.find(line));
 				openedItemsCount += 1;
@@ -222,8 +251,7 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 		REQUIRE(openedItemsCount == maxItemsToOpen);
 	}
 
-	SECTION("unread < max-browser-tabs")
-	{
+	SECTION("unread < max-browser-tabs") {
 		int maxItemsToOpen = 9;
 		int openedItemsCount = 0;
 		cfg.set_configvalue(
@@ -231,12 +259,12 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 
 		itemlist.process_op(OP_OPENALLUNREADINBROWSER);
 
-		std::ifstream browserFileStream(browserfile.getPath());
+		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
 			while (std::getline(browserFileStream, line)) {
 				INFO("Each URL should be present exactly once. "
-				     "Erase urls after first match to fail if "
-				     "an item opens twice.")
+					"Erase urls after first match to fail if "
+					"an item opens twice.")
 				REQUIRE(url_set.count(line) == 1);
 				url_set.erase(url_set.find(line));
 				openedItemsCount += 1;
@@ -255,12 +283,12 @@ TEST_CASE(
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
 	std::unordered_set<std::string> url_set;
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string line;
 	int itemCount = 6;
 
 	ConfigContainer cfg;
-	cfg.set_configvalue("browser", "echo %u >> " + browserfile.getPath());
+	cfg.set_configvalue("browser", "echo %u >> " + browserfile.get_path());
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
@@ -282,8 +310,7 @@ TEST_CASE(
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
 	itemlist.set_feed(feed);
 
-	SECTION("unread >= max-browser-tabs")
-	{
+	SECTION("unread >= max-browser-tabs") {
 		int maxItemsToOpen = 4;
 		int openedItemsCount = 0;
 		cfg.set_configvalue(
@@ -291,12 +318,12 @@ TEST_CASE(
 
 		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK);
 
-		std::ifstream browserFileStream(browserfile.getPath());
+		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
 			while (std::getline(browserFileStream, line)) {
 				INFO("Each URL should be present exactly once. "
-				     "Erase urls after first match to fail if "
-				     "an item opens twice.")
+					"Erase urls after first match to fail if "
+					"an item opens twice.")
 				REQUIRE(url_set.count(line) == 1);
 				url_set.erase(url_set.find(line));
 				openedItemsCount += 1;
@@ -307,8 +334,7 @@ TEST_CASE(
 			itemCount - maxItemsToOpen);
 	}
 
-	SECTION("unread < max-browser-tabs")
-	{
+	SECTION("unread < max-browser-tabs") {
 		int maxItemsToOpen = 9;
 		int openedItemsCount = 0;
 		cfg.set_configvalue(
@@ -316,12 +342,12 @@ TEST_CASE(
 
 		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK);
 
-		std::ifstream browserFileStream(browserfile.getPath());
+		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
 			while (std::getline(browserFileStream, line)) {
 				INFO("Each URL should be present exactly once. "
-				     "Erase urls after first match to fail if "
-				     "an item opens twice.")
+					"Erase urls after first match to fail if "
+					"an item opens twice.")
 				REQUIRE(url_set.count(line) == 1);
 				url_set.erase(url_set.find(line));
 				openedItemsCount += 1;
@@ -341,7 +367,7 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	FilterContainer filters;
 	TestHelpers::TempFile urlFile;
 
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string test_title = "Article Title";
 	std::string test_author = "Article Author";
 	std::string test_description = "Article Description";
@@ -365,16 +391,15 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	item->set_pubDate(test_pubDate);
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
 
-	SECTION("with external-url-viewer")
-	{
+	SECTION("with external-url-viewer") {
 		feed->add_item(item);
 		itemlist.set_feed(feed);
 		cfg.set_configvalue(
-			"external-url-viewer", "tee > " + urlFile.getPath());
+			"external-url-viewer", "tee > " + urlFile.get_path());
 
 		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
 
-		TestHelpers::AssertArticleFileContent(urlFile.getPath(),
+		TestHelpers::AssertArticleFileContent(urlFile.get_path(),
 			test_title,
 			test_author,
 			test_pubDate_str,
@@ -382,15 +407,13 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 			test_description);
 	}
 
-	SECTION("internal url viewer")
-	{
+	SECTION("internal url viewer") {
 		feed->add_item(item);
 		itemlist.set_feed(feed);
 		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
 	}
 
-	SECTION("no feed in formaction")
-	{
+	SECTION("no feed in formaction") {
 		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
 	}
 }
@@ -407,7 +430,7 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 	std::string line;
 	std::vector<std::string> bookmark_args;
 
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string test_title = "Article Title";
 	std::string feed_title = "Feed Title";
 	std::string separator = " ";
@@ -429,17 +452,17 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 	itemlist.set_feed(feed);
 
 	cfg.set_configvalue(
-		"bookmark-cmd", "echo > " + bookmarkFile.getPath());
+		"bookmark-cmd", "echo > " + bookmarkFile.get_path());
 
 	bookmark_args.push_back(extra_arg);
 	REQUIRE_NOTHROW(itemlist.process_op(OP_BOOKMARK, true, &bookmark_args));
 
-	std::ifstream browserFileStream(bookmarkFile.getPath());
+	std::ifstream browserFileStream(bookmarkFile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
 	REQUIRE(line ==
 		test_url + separator + test_title + separator + extra_arg +
-			separator + feed_title);
+		separator + feed_title);
 }
 
 TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
@@ -464,8 +487,7 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
-	SECTION("Single flag")
-	{
+	SECTION("Single flag") {
 		std::string flags = "G";
 		op_args.push_back(flags);
 
@@ -474,8 +496,7 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 		REQUIRE(item->flags() == flags);
 	}
 
-	SECTION("Unordered flags")
-	{
+	SECTION("Unordered flags") {
 		std::string flags = "abdefc";
 		std::string ordered_flags = "abcdef";
 		op_args.push_back(flags);
@@ -485,8 +506,7 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 		REQUIRE(item->flags() == ordered_flags);
 	}
 
-	SECTION("Duplicate flag in argument")
-	{
+	SECTION("Duplicate flag in argument") {
 		std::string flags = "Abd";
 		op_args.push_back(flags + "ddd");
 
@@ -495,38 +515,33 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 		REQUIRE(item->flags() == flags);
 	}
 
-	SECTION("Unauthorized values in arguments")
-	{
+	SECTION("Unauthorized values in arguments") {
 		std::string flags = "Abd";
-		SECTION("Numbers")
-		{
+		SECTION("Numbers") {
 			op_args.push_back(flags + "1236");
 
 			REQUIRE_NOTHROW(itemlist.process_op(
-				OP_EDITFLAGS, true, &op_args));
+					OP_EDITFLAGS, true, &op_args));
 			REQUIRE(item->flags() == flags);
 		}
-		SECTION("Symbols")
-		{
+		SECTION("Symbols") {
 			op_args.push_back(flags +
 				"%^\\*;\'\"&~#{([-|`_/@)]=}$£€µ,;:!?./§");
 
 			REQUIRE_NOTHROW(itemlist.process_op(
-				OP_EDITFLAGS, true, &op_args));
+					OP_EDITFLAGS, true, &op_args));
 			REQUIRE(item->flags() == flags);
 		}
-		SECTION("Accents")
-		{
+		SECTION("Accents") {
 			op_args.push_back(flags + "¨^");
 
 			REQUIRE_NOTHROW(itemlist.process_op(
-				OP_EDITFLAGS, true, &op_args));
+					OP_EDITFLAGS, true, &op_args));
 			REQUIRE(item->flags() == flags);
 		}
 	}
 
-	SECTION("All possible flags at once")
-	{
+	SECTION("All possible flags at once") {
 		std::string flags =
 			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
 		op_args.push_back(flags);
@@ -548,9 +563,9 @@ TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 	FilterContainer filters;
 
 	std::vector<std::string> op_args;
-	op_args.push_back(saveFile.getPath());
+	op_args.push_back(saveFile.get_path());
 
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string test_title = "Article Title";
 	std::string test_author = "Article Author";
 	std::string test_description = "Article Description";
@@ -580,7 +595,7 @@ TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 
 	REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, true, &op_args));
 
-	TestHelpers::AssertArticleFileContent(saveFile.getPath(),
+	TestHelpers::AssertArticleFileContent(saveFile.get_path(),
 		test_title,
 		test_author,
 		test_pubDate_str,
@@ -654,7 +669,7 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREVIOUS",
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	cfg.set_configvalue(
-		"external-url-viewer", "tee > " + articleFile.getPath());
+		"external-url-viewer", "tee > " + articleFile.get_path());
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
 	std::string line;
@@ -688,7 +703,7 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREVIOUS",
 	REQUIRE_NOTHROW(itemlist.process_op(OP_NEXT));
 	itemlist.process_op(OP_SHOWURLS);
 
-	std::ifstream fileStream(articleFile.getPath());
+	std::ifstream fileStream(articleFile.get_path());
 	std::getline(fileStream, line);
 	REQUIRE(line == prefix_title + second_article_title);
 
@@ -726,19 +741,17 @@ TEST_CASE("OP_TOGGLESHOWREAD switches the value of show-read-articles",
 	itemlist.set_feed(feed);
 	v.push_itemlist(feed);
 
-	SECTION("True to False")
-	{
+	SECTION("True to False") {
 		v.get_cfg()->set_configvalue("show-read-articles", "yes");
 		REQUIRE_NOTHROW(itemlist.process_op(OP_TOGGLESHOWREAD));
 		REQUIRE_FALSE(v.get_cfg()->get_configvalue_as_bool(
-			"show-read-articles"));
+				"show-read-articles"));
 	}
-	SECTION("False to True")
-	{
+	SECTION("False to True") {
 		v.get_cfg()->set_configvalue("show-read-articles", "no");
 		REQUIRE_NOTHROW(itemlist.process_op(OP_TOGGLESHOWREAD));
 		REQUIRE(v.get_cfg()->get_configvalue_as_bool(
-			"show-read-articles"));
+				"show-read-articles"));
 	}
 }
 
@@ -753,9 +766,9 @@ TEST_CASE("OP_PIPE_TO pipes an article's content to an external command",
 	FilterContainer filters;
 
 	std::vector<std::string> op_args;
-	op_args.push_back("tee > " + articleFile.getPath());
+	op_args.push_back("tee > " + articleFile.get_path());
 
-	std::string test_url = "http://test_url";
+	const std::string test_url = "http://test_url";
 	std::string test_title = "Article Title";
 	std::string test_author = "Article Author";
 	std::string test_description = "Article Description";
@@ -785,7 +798,7 @@ TEST_CASE("OP_PIPE_TO pipes an article's content to an external command",
 
 	REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, true, &op_args));
 
-	TestHelpers::AssertArticleFileContent(articleFile.getPath(),
+	TestHelpers::AssertArticleFileContent(articleFile.get_path(),
 		test_title,
 		test_author,
 		test_pubDate_str,
