@@ -1,9 +1,9 @@
 #define CATCH_CONFIG_RUNNER
 #include "3rd-party/catch.hpp"
 
-#include <logger.h>
+#include "logger.h"
 
-int main(int argc, char *argv[])
+int main(int argc, char* argv[])
 {
 	setlocale(LC_CTYPE, "");
 
@@ -11,4 +11,3 @@ int main(int argc, char *argv[])
 
 	return Catch::Session().run(argc, argv);
 }
-
